"""
"""
import pytest

from ArWikiCats.format_bots.pf_keys import change_key_mappings_replacements

data1 = {
    "africa cup of nations": "africa cup-of-nations",
    "african-american muslims": "africanamerican muslims",
    "amphibious warfare vessels of islamic republic of iran navy": "amphibious warfare vessels of islamic republic-of iran navy",
    "auxiliary ships of islamic republic of iran navy": "auxiliary ships of islamic republic-of iran navy",
    "category:2016 women's africa cup of nations squad navigational boxes": "category:2016 women's africa cup-of-nations squad navigational boxes",
    "category:2016 women's africa cup of nations": "category:2016 women's africa cup-of-nations",
    "category:2017 american television seasons": "category:2017 american television-seasons",
    "category:2017 american television series debuts": "category:2017 american television series-debuts",
    "category:2017 american television series endings": "category:2017 american television series-endings",
    "category:2018 women's africa cup of nations squad navigational boxes": "category:2018 women's africa cup-of-nations squad navigational boxes",
    "category:2018 women's africa cup of nations": "category:2018 women's africa cup-of-nations",
    "category:2022 women's africa cup of nations players": "category:2022 women's africa cup-of-nations players",
    "category:2022 women's africa cup of nations squad navigational boxes": "category:2022 women's africa cup-of-nations squad navigational boxes",
    "category:2022 women's africa cup of nations": "category:2022 women's africa cup-of-nations",
    "category:2024 women's africa cup of nations players": "category:2024 women's africa cup-of-nations players",
    "category:2024 women's africa cup of nations": "category:2024 women's africa cup-of-nations",
    "category:adaptations of works by greek writers": "category:adaptations-of-works by greek writers",
    "category:adaptations of works by irish writers": "category:adaptations-of-works by irish writers",
    "category:adaptations of works by italian writers": "category:adaptations-of-works by italian writers",
    "category:african-american coaches of canadian football": "category:africanamerican coaches of canadian-football",
    "category:african-american history by state": "category:africanamerican history by state",
    "category:african-american non-fiction writers": "category:africanamerican non-fiction writers",
    "category:ambassadors of federated states of micronesia in yemen by year": "category:ambassadors of federated states-of micronesia in yemen by year",
    "category:american players of canadian football by populated place": "category:american players of canadian-football by populated place",
    "category:american players of canadian football by state": "category:american players of canadian-football by state",
    "category:american players of canadian football": "category:american players of canadian-football",
    "category:australian players of canadian football": "category:australian players of canadian-football",
    "category:bahamian players of canadian football": "category:bahamian players of canadian-football",
    "category:bodies of water of carmarthenshire": "category:bodies-of-water of carmarthenshire",
    "category:bodies of water of germany by state": "category:bodies-of-water of germany by state",
    "category:cameroonian players of canadian football": "category:cameroonian players of canadian-football",
    "category:canadian football by city": "category:canadian-football by city",
    "category:canadian football by country": "category:canadian-football by country",
    "category:canadian football people from windsor, ontario": "category:canadian-football people from windsor, ontario",
    "category:canadian football people from winnipeg": "category:canadian-football people from winnipeg",
    "category:canadian football people": "category:canadian-football people",
    "category:canadian football placekickers": "category:canadian-football placekickers",
    "category:canadian football players in edmonton": "category:canadian-football playerss in edmonton",
    "category:canadian football quarterbacks": "category:canadian-football quarterbacks",
    "category:canadian football running backs": "category:canadian-football running backs",
    "category:canadian football scouts": "category:canadian-football scouts",
    "category:cuban players of canadian football": "category:cuban players of canadian-football",
    "category:deaths by american airstrikes during world war ii": "category:deaths by american airstrikes during world-war-ii",
    "category:deaths by car bomb by country": "category:deaths by-car-bomb by country",
    "category:deaths by car bomb in afghanistan": "category:deaths by-car-bomb in afghanistan",
    "category:deaths by car bomb in england": "category:deaths by-car-bomb in england",
    "category:deaths by firearm in afghanistan": "category:deaths by-firearm-in afghanistan",
    "category:deaths by firearm in alabama": "category:deaths by-firearm-in alabama",
    "category:deaths by firearm in alaska": "category:deaths by-firearm-in alaska",
    "category:deaths by firearm in wisconsin": "category:deaths by-firearm-in wisconsin",
    "category:deaths by firearm in yugoslavia": "category:deaths by-firearm-in yugoslavia",
    "category:deaths by firearm in zimbabwe": "category:deaths by-firearm-in zimbabwe",
    "category:deaths from brain cancer in northern ireland": "category:deaths from brain cancer in northern-ireland",
    "category:deaths from brain cancer in republic of ireland": "category:deaths from brain cancer in republic-of ireland",
    "category:deaths from cancer in federated states of micronesia": "category:deaths from cancer in federated states-of micronesia",
    "category:deaths from cancer in isle of man": "category:deaths from cancer in isle-of man",
    "category:deaths from cancer in northern ireland": "category:deaths from cancer in northern-ireland",
    "category:deaths from cancer in people's republic of china": "category:deaths from cancer in people's republic-of china",
    "category:deaths from cancer in republic of ireland": "category:deaths from cancer in republic-of ireland",
    "category:deaths from covid-19 pandemic in northern ireland": "category:deaths from covid-19 pandemic in northern-ireland",
    "category:deaths from lung cancer in northern ireland": "category:deaths from lung cancer in northern-ireland",
    "category:deaths from lung cancer in people's republic of china": "category:deaths from lung cancer in people's republic-of china",
    "category:deaths from lung cancer in republic of ireland": "category:deaths from lung cancer in republic-of ireland",
    "category:deaths from pneumonia in people's republic of china": "category:deaths from pneumonia in people's republic-of china",
    "category:deaths from pneumonia in republic of ireland": "category:deaths from pneumonia in republic-of ireland",
    "category:democratic-republic-of-congo players of canadian football": "category:democratic-republic-of-congo players of canadian-football",
    "category:dominican republic players of canadian football": "category:dominican republic players of canadian-football",
    "category:english players of canadian football": "category:english players of canadian-football",
    "category:expatriate women's futsal players in kuwait": "category:expatriate women's futsal playerss in kuwait",
    "category:expatriate women's futsal players in maldives": "category:expatriate women's futsal playerss in maldives",
    "category:female handball players in turkey by club": "category:female handball playerss in turkey by club",
    "category:georgia (u.s. state) attorney general elections": "category:georgia (u.s. state) attorney general-elections",
    "category:georgia (u.s. state) labor commissioners": "category:georgia (u.s. state) labour commissioners",
    "category:georgia (u.s. state) presidential primaries": "category:georgia (u.s. state) presidential-primaries",
    "category:guyanese players of canadian football": "category:guyanese players of canadian-football",
    "category:honduran players of canadian football": "category:honduran players of canadian-football",
    "category:jamaican players of canadian football": "category:jamaican players of canadian-football",
    "category:lebanese players of canadian football": "category:lebanese players of canadian-football",
    "category:ministers for foreign affairs of abkhazia": "category:ministers-for foreign affairs of abkhazia",
    "category:ministers for foreign affairs of luxembourg": "category:ministers-for foreign affairs of luxembourg",
    "category:ministers for foreign affairs of singapore": "category:ministers-for foreign affairs of singapore",
    "category:ministers for internal affairs of abkhazia": "category:ministers-for internal affairs of abkhazia",
    "category:ministers for public works of luxembourg": "category:ministers-for public works of luxembourg",
    "category:ministers of labour and social security of turkey": "category:ministers of labour-and-social security of turkey",
    "category:national register of historic places in westchester county, new york": "category:national-register-of-historic-places in westchester county, new york",
    "category:nigerian players of canadian football": "category:nigerian players of canadian-football",
    "category:non-profit organizations based in northern ireland": "category:non-profit organizations based in northern-ireland",
    "category:non-profit organizations based in republic of ireland": "category:non-profit organizations based in republic-of ireland",
    "category:panamanian players of canadian football": "category:panamanian players of canadian-football",
    "category:players of canadian football by nationality": "category:players of canadian-football by nationality",
    "category:players of canadian football by position": "category:players of canadian-football by position",
    "category:players of canadian football": "category:players of canadian-football",
    "category:puerto rican players of canadian football": "category:puerto rican players of canadian-football",
    "category:republic of ireland association football leagues": "category:republic-of ireland association football leagues",
    "category:republic of ireland women's international rugby union players": "category:republic-of ireland women's international rugby union players",
    "category:republic of ireland women's youth international footballers": "category:republic-of ireland women's youth international footballers",
    "category:samoan players of canadian football": "category:samoan players of canadian-football",
    "category:saudi arabian non-fiction writers": "category:saudiarabian non-fiction writers",
    "category:secretaries-of labor and employment of philippines": "category:secretaries-of labour and employment of philippines",
    "category:secretaries-of labor of mexico": "category:secretaries-of labour of mexico",
    "category:sierra leonean players of canadian football": "category:sierra leonean players of canadian-football",
    "category:television series endings by year": "category:television series-endings by year",
    "category:united states deputy secretaries-of labor": "category:united states deputy secretaries-of labour",
    "category:united states secretaries-of labor": "category:united states secretaries-of labour",
    "category:venezuelan players of canadian football": "category:venezuelan players of canadian-football",
    "category:wheelchair basketball players in 2020 parapan american games": "category:wheelchair basketball playerss in 2020 parapan american games",
    "category:wheelchair basketball players in 2020 summer paralympics": "category:wheelchair basketball playerss in 2020 summer paralympics",
    "category:wheelchair basketball players in turkey by team": "category:wheelchair basketball playerss in turkey by team",
    "category:wheelchair rugby players in 2020 parapan american games": "category:wheelchair rugby playerss in 2020 parapan american games",
    "category:wheelchair rugby players in 2020 summer paralympics": "category:wheelchair rugby playerss in 2020 summer paralympics",
    "category:wheelchair tennis players in 2020 asian para games": "category:wheelchair tennis playerss in 2020 asian para games",
    "category:wheelchair tennis players in 2020 parapan american games": "category:wheelchair tennis playerss in 2020 parapan american games",
    "category:wheelchair tennis players in 2020 summer paralympics": "category:wheelchair tennis playerss in 2020 summer paralympics",
    "category:women singers from kingdom of prussia": "category:women singers from kingdom-of prussia",
    "category:women's africa cup of nations players": "category:women's africa cup-of-nations players",
    "category:women's africa cup of nations qualification": "category:women's africa cup-of-nations qualification",
    "category:women's africa cup of nations tournaments": "category:women's africa cup-of-nations tournaments",
    "category:women's africa cup of nations": "category:women's africa cup-of-nations",
    "destroyers of islamic republic of iran navy": "destroyers of islamic republic-of iran navy",
    "frigates of islamic republic of iran navy": "frigates of islamic republic-of iran navy",
    "kingdom of hungary in middle ages": "kingdom-of hungary in middle ages",
    "mine warfare vessels of islamic republic of iran navy": "mine warfare vessels of islamic republic-of iran navy",
    "missile boats of islamic republic of iran navy": "missile boats of islamic republic-of iran navy",
    "saudi arabian muslims": "saudiarabian muslims",
    "ship classes of islamic republic of iran navy": "ship classes of islamic republic-of iran navy",
    "ships of islamic republic of iran navy": "ships of islamic republic-of iran navy",
    "submarines of islamic republic of iran navy": "submarines of islamic republic-of iran navy",
    "women in federated states of micronesia": "women in federated states-of micronesia",
}

data2 = {
    "archery at 2004 summer olympics": "archery in 2004 summer olympics",
    "category:100 metres at african championships in athletics": "category:100 metres in african championships in athletics",
    "category:100 metres at iaaf world youth championships in athletics": "category:100 metres in iaaf world youth championships in athletics",
    "category:100 metres at world para athletics championships": "category:100 metres in world para athletics championships",
    "category:argentina at universiade": "category:argentina in universiade",
    "category:argentina at winter olympics": "category:argentina in winter olympics",
    "category:arizona house of representatives": "category:arizona house-of-representatives",
    "category:athletics at summer universiade navigational boxes": "category:athletics in summer universiade navigational boxes",
    "category:athletics at universiade navigational boxes": "category:athletics in universiade navigational boxes",
    "category:australia at summer universiade": "category:australia in summer universiade",
    "category:cape verde at paralympics": "category:cape verde in paralympics",
    "category:cross-country skiers at 1992 winter paralympics": "category:cross-country skiers in 1992 winter paralympics",
    "category:equatoguinean women's footballers": "category:equatoguinean female footballers",
    "category:expatriate association football managers by country of residence": "category:expatriate association football managers by country-of-residence",
    "category:figure skaters at 2002 winter olympics": "category:figure skaters in 2002 winter olympics",
    "category:figure skaters at 2003 asian winter games": "category:figure skaters in 2003 asian winter games",
    "category:figure skaters at 2007 winter universiade": "category:figure skaters in 2007 winter universiade",
    "category:film set at metropolitan museum of art": "category:film set in metropolitan museum of art",
    "category:game of thrones (season 1) episodes": "category:game-of-thrones (season 1) episodes",
    "category:georgia (u.s. state) house of representatives": "category:georgia (u.s. state) house-of-representatives",
    "category:internees at sheberghan prison": "category:internees in sheberghan prison",
    "category:iowa house of representatives": "category:iowa house-of-representatives",
    "category:montana house of representatives": "category:montana house-of-representatives",
    "category:multi-sport events at yemen": "category:multi-sport events in yemen",
    "category:nations at 2010 summer youth olympics": "category:nations in 2010 summer youth olympics",
    "category:nations at universiade": "category:nations in universiade",
    "category:nebraska house of representatives": "category:nebraska house-of-representatives",
    "category:new mexico house of representatives": "category:new mexico house-of-representatives",
    "category:roller skaters at 2003 pan american games": "category:roller skaters in 2003 pan american games",
    "category:schools for deaf in new york (state)": "category:schools for-deaf in new york (state)",
    "category:ski jumpers at 2007 winter universiade": "category:ski jumpers in 2007 winter universiade",
    "category:ski jumping at winter universiade": "category:ski jumping in winter universiade",
    "category:sports at summer universiade": "category:sports in summer universiade",
    "category:water polo at summer universiade": "category:water polo in summer universiade",
    "category:wheelchair basketball at 2020 asean para games": "category:wheelchair basketball in 2020 asean para games",
    "category:wheelchair basketball at 2020 parapan american games": "category:wheelchair basketball in 2020 parapan american games",
    "category:wheelchair basketball at 2020 summer paralympics": "category:wheelchair basketball in 2020 summer paralympics",
    "category:wheelchair basketball at asean para games": "category:wheelchair basketball in asean para games",
    "category:wheelchair basketball at asian para games": "category:wheelchair basketball in asian para games",
    "category:wheelchair basketball at multi-sport events": "category:wheelchair basketball in multi-sport events",
    "category:wheelchair basketball at parapan american games": "category:wheelchair basketball in parapan american games",
    "category:wheelchair basketball at summer paralympics navigational boxes": "category:wheelchair basketball in summer paralympics navigational boxes",
    "category:wheelchair basketball at summer paralympics": "category:wheelchair basketball in summer paralympics",
    "category:wheelchair basketball players at 2020 parapan american games": "category:wheelchair basketball players in 2020 parapan american games",
    "category:wheelchair basketball players at 2020 summer paralympics": "category:wheelchair basketball players in 2020 summer paralympics",
    "category:wheelchair curlers at 2020 winter paralympics": "category:wheelchair curlers in 2020 winter paralympics",
    "category:wheelchair curling at 2020 winter paralympics": "category:wheelchair curling in 2020 winter paralympics",
    "category:wheelchair curling at winter paralympics": "category:wheelchair curling in winter paralympics",
    "category:wheelchair fencers at 2020 summer paralympics": "category:wheelchair fencers in 2020 summer paralympics",
    "category:wheelchair fencing at 2020 asean para games": "category:wheelchair fencing in 2020 asean para games",
    "category:wheelchair fencing at 2020 summer paralympics": "category:wheelchair fencing in 2020 summer paralympics",
    "category:wheelchair fencing at asean para games": "category:wheelchair fencing in asean para games",
    "category:wheelchair fencing at summer paralympics navigational boxes": "category:wheelchair fencing in summer paralympics navigational boxes",
    "category:wheelchair fencing at summer paralympics": "category:wheelchair fencing in summer paralympics",
    "category:wheelchair racers at 2020 summer olympics": "category:wheelchair racers in 2020 summer olympics",
    "category:wheelchair racing at summer olympics": "category:wheelchair racing in summer olympics",
    "category:wheelchair rugby at 2020 parapan american games": "category:wheelchair rugby in 2020 parapan american games",
    "category:wheelchair rugby at 2020 summer paralympics": "category:wheelchair rugby in 2020 summer paralympics",
    "category:wheelchair rugby at 2020 world games": "category:wheelchair rugby in 2020 world games",
    "category:wheelchair rugby at parapan american games": "category:wheelchair rugby in parapan american games",
    "category:wheelchair rugby at summer paralympics": "category:wheelchair rugby in summer paralympics",
    "category:wheelchair rugby at world games": "category:wheelchair rugby in world games",
    "category:wheelchair rugby players at 2020 parapan american games": "category:wheelchair rugby players in 2020 parapan american games",
    "category:wheelchair rugby players at 2020 summer paralympics": "category:wheelchair rugby players in 2020 summer paralympics",
    "category:wheelchair tennis at 2020 asean para games": "category:wheelchair tennis in 2020 asean para games",
    "category:wheelchair tennis at 2020 parapan american games": "category:wheelchair tennis in 2020 parapan american games",
    "category:wheelchair tennis at 2020 summer paralympics": "category:wheelchair tennis in 2020 summer paralympics",
    "category:wheelchair tennis at asean para games": "category:wheelchair tennis in asean para games",
    "category:wheelchair tennis at asian para games": "category:wheelchair tennis in asian para games",
    "category:wheelchair tennis at multi-sport events": "category:wheelchair tennis in multi-sport events",
    "category:wheelchair tennis at parapan american games": "category:wheelchair tennis in parapan american games",
    "category:wheelchair tennis at summer paralympics navigational boxes": "category:wheelchair tennis in summer paralympics navigational boxes",
    "category:wheelchair tennis at summer paralympics": "category:wheelchair tennis in summer paralympics",
    "category:wheelchair tennis players at 2020 asian para games": "category:wheelchair tennis players in 2020 asian para games",
    "category:wheelchair tennis players at 2020 parapan american games": "category:wheelchair tennis players in 2020 parapan american games",
    "category:wheelchair tennis players at 2020 summer paralympics": "category:wheelchair tennis players in 2020 summer paralympics",
    "category:wisconsin house of representatives": "category:wisconsin house-of-representatives",
    "first league of serbia and montenegro": "first league of serbia-and-montenegro",
    "synchronized swimming at 2004 summer olympics": "synchronized swimming in 2004 summer olympics",
    "taekwondo at 2004 summer olympics": "taekwondo in 2004 summer olympics",
    "taekwondo at 2021 islamic solidarity games": "taekwondo in 2021 islamic solidarity games",
    "taekwondo at islamic solidarity games": "taekwondo in islamic solidarity games",
    "tour de france": "tour of france",
    "triathlon at 2004 summer olympics": "triathlon in 2004 summer olympics",
    "water polo at world aquatics championships": "water polo in world aquatics championships",
    "wrestling at 2004 summer olympics": "wrestling in 2004 summer olympics",
}


based_data = {
    "category:american films based on british films": "category:american films basedon british films",
    "category:lithuanian films based on non-lithuanian films": "category:lithuanian films basedon non-lithuanian films",
    "category:malaysian films based on non-malaysian films": "category:malaysian films basedon non-malaysian films",
    "category:mexican films based on non-mexican films": "category:mexican films basedon non-mexican films",
    "category:non-american films based on american films": "category:non-american films basedon american films",
    "category:non-argentine films based on argentine films": "category:non-argentine films basedon argentine films",
    "category:non-australian films based on australian films": "category:non-australian films basedon australian films",
    "category:non-british films based on british films": "category:non-british films basedon british films",
    "category:non-canadian films based on canadian films": "category:non-canadian films basedon canadian films",
    "category:non-colombian films based on colombian films": "category:non-colombian films basedon colombian films",
    "category:non-french films based on french films": "category:non-french films basedon french films",
    "category:vietnamese films based on non-vietnamese films": "category:vietnamese films basedon non-vietnamese films",
    "category:works based on non-fiction books": "category:works basedon non-fiction books",
}


@pytest.mark.parametrize("category, expected", data1.items(), ids=data1.keys())
def test_pf_keys_1(category: str, expected: str) -> None:
    assert change_key_mappings_replacements(category) == expected


@pytest.mark.parametrize("category, expected", based_data.items(), ids=based_data.keys())
def test_based_data(category: str, expected: str) -> None:
    assert change_key_mappings_replacements(category) == expected



"""
Tests
"""

from typing import Callable

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats.new_resolvers.countries_names_resolvers.geo_names_formats import resolve_by_geo_names
from ArWikiCats.new_resolvers.reslove_all import new_resolvers_all

data_0 = {
    "foreign relations of fatimid caliphate": "علاقات الدولة الفاطمية الخارجية",
    "Banks of bologna": "بنوك بولونيا",
    "Military history of Amhara Region": "تاريخ أمهرة العسكري",
    "Military history of Bologna": "تاريخ بولونيا العسكري",
    "Military history of Hubei": "تاريخ خوبي العسكري",
    "Military history of republic of Venice": "تاريخ جمهورية البندقية العسكري",
    "Military history of the Tsardom of Russia": "تاريخ روسيا القيصرية العسكري",
    "Military history of West Virginia": "تاريخ فرجينيا الغربية العسكري",
    "Political history of Kurdistan": "تاريخ كردستان السياسي",
    "Political history of Manitoba": "تاريخ مانيتوبا السياسي",
    "Political history of Tamil Nadu": "تاريخ تامل نادو السياسي",
    "Political history of Texas": "تاريخ تكساس السياسي",
    "Political history of West Virginia": "تاريخ فرجينيا الغربية السياسي",
}

data_1 = {}


@pytest.mark.parametrize("category, expected", data_1.items(), ids=data_1.keys())
@pytest.mark.fast
def test_geo_data_0(category: str, expected: str) -> None:
    label = resolve_by_geo_names(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data_0.items(), ids=data_0.keys())
@pytest.mark.fast
def test_geo_data_1(category: str, expected: str) -> None:
    label = resolve_by_geo_names(category)
    assert label == expected


TEMPORAL_CASES = [
    ("test_geo_data_0", data_0, resolve_by_geo_names),
    ("test_geo_data_1", data_1, resolve_by_geo_names),
]


@pytest.mark.dump
@pytest.mark.parametrize("name,data, callback", TEMPORAL_CASES)
def test_all_dump(name: str, data: dict[str, str], callback: Callable) -> None:
    expected, diff_result = one_dump_test(data, callback)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""Unit tests for :mod:`ArWikiCats.translations.jobs.jobs_defs`."""

from __future__ import annotations

from typing import Any, Mapping

import pytest

from ArWikiCats.translations.jobs import jobs_defs


def test_join_terms_trims_and_skips_empty_strings() -> None:
    """``join_terms`` should normalise whitespace and ignore empty values."""

    result = jobs_defs.join_terms("  leading", "", "middle  ", " trailing ")
    assert result == "leading middle trailing"


def test_combine_gendered_labels_respects_require_base_womens() -> None:
    """When ``require_base_womens`` is set the feminine label may remain blank."""

    base = {"males": "راهب", "females": ""}
    suffix = {"males": "كاثوليكي", "females": "كاثوليكية"}
    combined = jobs_defs.combine_gendered_labels(base, suffix, require_base_womens=True)
    assert combined == {"males": "راهب كاثوليكي", "females": ""}


def test_merge_gendered_maps_copies_source_values() -> None:
    """Merged mappings should copy the source entries to avoid shared state."""

    target = {"key": {"males": "أ", "females": "ب"}}
    source_value = {"males": "ج", "females": "د"}
    source = {"key": source_value}

    jobs_defs.merge_gendered_maps(target, source)

    assert target["key"] == source_value
    assert target["key"] is not source_value


def test_load_gendered_label_map_filters_invalid_entries(monkeypatch: pytest.MonkeyPatch) -> None:
    """``load_gendered_label_map`` should ignore non-mapping or malformed items."""

    def fake_loader(filename: str) -> Mapping[str, Any]:
        return {
            "valid": {"males": "لاعب", "females": "لاعبة"},
            "missing": {"males": "", "females": ""},
            # Non-mapping and non-string entries should be skipped silently.
            "bad": ["not", "a", "mapping"],
            10: {"males": "number", "females": "number"},
        }

    monkeypatch.setattr(jobs_defs, "open_json_file", fake_loader)

    result = jobs_defs.load_gendered_label_map("ignored")

    assert result == {
        "valid": {"males": "لاعب", "females": "لاعبة"},
        "missing": {"males": "", "females": ""},
    }



"""
Utility tables for translating multi-season sporting competitions.
"""

from ...helps import len_print

BASE_GAME_LABELS = {
    "african games": "الألعاب الإفريقية",
    "all-africa games": "ألعاب عموم إفريقيا",
    "asian games": "الألعاب الآسيوية",
    "central american games": "ألعاب أمريكا الوسطى",
    "commonwealth games": "ألعاب الكومنولث",
    "deaflympic games": "ألعاب ديفلمبياد",
    "european youth olympic winter": "الألعاب الأولمبية الشبابية الأوروبية الشتوية",
    "european youth olympic": "الألعاب الأولمبية الشبابية الأوروبية",
    "jeux de la francophonie": "الألعاب الفرانكوفونية",
    "olympic games": "الألعاب الأولمبية",
    "olympics": "الألعاب الأولمبية",
    "paralympics games": "الألعاب البارالمبية",
    "south american games": "ألعاب أمريكا الجنوبية",
    "universiade": "الألعاب الجامعية",
    "world games": "دورة الألعاب العالمية",
    "youth olympic games": "الألعاب الأولمبية الشبابية",
    "youth olympics": "الألعاب الأولمبية الشبابية",
}


SUMMER_WINTER_GAMES = {
    "african games": "الألعاب الإفريقية",
    "asian beach games": "دورة الألعاب الآسيوية الشاطئية",
    "asian games": "الألعاب الآسيوية",
    "asian para games": "الألعاب البارالمبية الآسيوية",
    "asian summer games": "الألعاب الآسيوية الصيفية",
    "asian winter games": "الألعاب الآسيوية الشتوية",
    "bolivarian games": "الألعاب البوليفارية",
    "central american and caribbean games": "ألعاب أمريكا الوسطى والكاريبي",
    "central american games": "ألعاب أمريكا الوسطى",
    "commonwealth games": "ألعاب الكومنولث",
    "commonwealth youth games": "ألعاب الكومنولث الشبابية",
    "european games": "الألعاب الأوروبية",
    "european youth olympic winter": "الألعاب الأولمبية الشبابية الأوروبية الشتوية",
    "european youth olympic": "الألعاب الأولمبية الشبابية الأوروبية",
    "fis nordic world ski championships": "بطولة العالم للتزلج النوردي على الثلج",
    "friendship games": "ألعاب الصداقة",
    "goodwill games": "ألعاب النوايا الحسنة",
    "islamic solidarity games": "ألعاب التضامن الإسلامي",
    "maccabiah games": "الألعاب المكابيه",
    "mediterranean games": "الألعاب المتوسطية",
    "micronesian games": "الألعاب الميكرونيزية",
    "military world games": "دورة الألعاب العسكرية",
    "asian indoor games": "دورة الألعاب الآسيوية داخل الصالات",
    "pan american games": "دورة الألعاب الأمريكية",
    "pan arab games": "دورة الألعاب العربية",
    "pan asian games": "دورة الألعاب الآسيوية",
    "paralympic": "الألعاب البارالمبية",
    "paralympics": "الألعاب البارالمبية",
    "parapan american games": "ألعاب بارابان الأمريكية",
    "sea games": "ألعاب البحر",
    "south american games": "ألعاب أمريكا الجنوبية",
    "south asian beach games": "دورة ألعاب جنوب أسيا الشاطئية",
    "south asian games": "ألعاب جنوب أسيا",
    "south asian winter games": "ألعاب جنوب آسيا الشتوية",
    "southeast asian games": "ألعاب جنوب شرق آسيا",
    "summer olympics": "الألعاب الأولمبية الصيفية",
    "summer universiade": "الألعاب الجامعية الصيفية",
    "summer world university games": "ألعاب الجامعات العالمية الصيفية",
    "the universiade": "الألعاب الجامعية",
    "universiade": "الألعاب الجامعية",
    "winter olympics": "الألعاب الأولمبية الشتوية",
    "winter universiade": "الألعاب الجامعية الشتوية",
    "winter world university games": "ألعاب الجامعات العالمية الشتوية",
    "world championships": "بطولات العالم",
    "youth olympic": "الألعاب الأولمبية الشبابية",
    "youth olympics games": "الألعاب الأولمبية الشبابية",
    "youth olympics": "الألعاب الأولمبية الشبابية",
    "deaflympic games": "ألعاب ديفلمبياد",
}


def _build_seasonal_labels() -> dict[str, str]:
    """Return the label table that includes seasonal variants."""

    seasonal_labels = dict(SUMMER_WINTER_GAMES)

    for base_key, base_label in BASE_GAME_LABELS.items():
        seasonal_labels[base_key] = base_label
        seasonal_labels[f"winter {base_key}"] = f"{base_label} الشتوية"
        seasonal_labels[f"summer {base_key}"] = f"{base_label} الصيفية"
        seasonal_labels[f"west {base_key}"] = f"{base_label} الغربية"
        seasonal_labels[f"east {base_key}"] = f"{base_label} الشرقية"

    return seasonal_labels


def _build_tab_labels(SEASONAL_GAME_LABELS) -> dict[str, str]:
    """Return tabs that combine games with category labels."""

    GAME_CATEGORY_LABELS = {
        "competitions": "منافسات",
        "events": "أحداث",
        "festival": "مهرجانات",
        "bids": "عروض",
        "templates": "قوالب",
    }

    tab_labels: dict[str, str] = {}

    for game_key, game_label in SEASONAL_GAME_LABELS.items():
        tab_labels[game_key] = game_label

        for category_key, category_label in GAME_CATEGORY_LABELS.items():
            category_entry_key = f"{game_key} {category_key}"
            tab_labels[category_entry_key] = f"{category_label} {game_label}"

    return tab_labels


SEASONAL_GAME_LABELS = _build_seasonal_labels()

SUMMER_WINTER_TABS = _build_tab_labels(SEASONAL_GAME_LABELS)

__all__ = [
    "SEASONAL_GAME_LABELS",
    "SUMMER_WINTER_GAMES",
    "SUMMER_WINTER_TABS",
]

len_print.data_len(
    "games_labs.py",
    {
        "SEASONAL_GAME_LABELS": SEASONAL_GAME_LABELS,
        "SUMMER_WINTER_GAMES": SUMMER_WINTER_GAMES,
        "SUMMER_WINTER_TABS": SUMMER_WINTER_TABS,
    },
)



"""
Tests
"""

import pytest
from load_one_data import dump_diff, dump_same_and_not_same, one_dump_test

from ArWikiCats.new.resolve_films_bots.film_keys_bot import Films
from ArWikiCats.new.resolve_films_bots.resolve_films_labels import _get_films_key_tyty_new

test_data3 = {
    "animated short film comics": "قصص مصورة رسوم متحركة قصيرة",
    "animated short film film series": "سلاسل أفلام رسوم متحركة قصيرة",
    "animated short film soap opera": "مسلسلات طويلة رسوم متحركة قصيرة",
    "animated short film television episodes": "حلقات تلفزيونية رسوم متحركة قصيرة",
    "animated short film television films": "أفلام تلفزيونية رسوم متحركة قصيرة",
    "animated short film television news": "أخبار تلفزيونية رسوم متحركة قصيرة",
    "animated short film television programmes": "برامج تلفزيونية رسوم متحركة قصيرة",
    "animated short film television programs": "برامج تلفزيونية رسوم متحركة قصيرة",
    "animated short film television series": "مسلسلات تلفزيونية رسوم متحركة قصيرة",
    "animated short film video games": "ألعاب فيديو رسوم متحركة قصيرة",
    "animated short film web series": "مسلسلات ويب رسوم متحركة قصيرة",
    "comic science fiction comics": "قصص مصورة خيالية علمية كوميدية",
    "comic science fiction film series": "سلاسل أفلام خيالية علمية كوميدية",
    "comic science fiction soap opera": "مسلسلات طويلة خيالية علمية كوميدية",
    "comic science fiction television episodes": "حلقات تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television films": "أفلام تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television news": "أخبار تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television programmes": "برامج تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television programs": "برامج تلفزيونية خيالية علمية كوميدية",
    "comic science fiction television series": "مسلسلات تلفزيونية خيالية علمية كوميدية",
    "comic science fiction video games": "ألعاب فيديو خيالية علمية كوميدية",
    "comic science fiction web series": "مسلسلات ويب خيالية علمية كوميدية",
    "television comics": "قصص مصورة تلفزيونية",
    "television episodes": "حلقات تلفزيونية",
    "television film series": "سلاسل أفلام تلفزيونية",
    "television films": "أفلام تلفزيونية",
    "television miniseries": "مسلسلات قصيرة تلفزيونية",
    "television news": "أخبار تلفزيونية",
    "television programmes": "برامج تلفزيونية",
    "television programs": "برامج تلفزيونية",
    "television series": "مسلسلات تلفزيونية",
    "television soap opera": "مسلسلات طويلة تلفزيونية",
    "television video games": "ألعاب فيديو تلفزيونية",
    "television web series": "مسلسلات ويب تلفزيونية",
    "comics": "قصص مصورة",
    "film series": "سلاسل أفلام",
    "soap opera": "مسلسلات طويلة",
    "video games": "ألعاب فيديو",
    "web series": "مسلسلات ويب",
}


@pytest.mark.parametrize("category, expected", test_data3.items(), ids=test_data3.keys())
@pytest.mark.fast
def test_Films(category: str, expected: str) -> None:
    label = Films(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", test_data3.items(), ids=test_data3.keys())
@pytest.mark.fast
def test_get_films_key_tyty_new(category: str, expected: str) -> None:
    label = _get_films_key_tyty_new(category)
    assert label == expected


to_test = [
    ("test_Films", test_data3, Films),
    ("test_Films_tyty", test_data3, _get_films_key_tyty_new),
]


@pytest.mark.parametrize("name,data,callback", to_test)
@pytest.mark.dump
def test_resolve_films_all(name: str, data: dict[str, str], callback) -> None:
    expected, diff_result = one_dump_test(data, callback)

    dump_diff(diff_result, name)
    dump_same_and_not_same(data, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""Aggregate translation tables for country and region labels."""

from __future__ import annotations

from collections.abc import Mapping, MutableMapping

from ...helps import len_print, logger
from ..mixed.all_keys2 import pf_keys2
from ..mixed.all_keys5 import BASE_POP_FINAL_5
from ..nats.Nationality import all_country_ar
from ..tax_table import Taxons_table as TAXON_TABLE
from ._shared import load_json_mapping
from .Cities import CITY_TRANSLATIONS_LOWER
from .labels_country2 import COUNTRY_ADMIN_LABELS
from .regions import MAIN_REGION_TRANSLATIONS
from .regions2 import INDIA_REGION_TRANSLATIONS, SECONDARY_REGION_TRANSLATIONS
from .us_counties import US_COUNTY_TRANSLATIONS

US_STATES = {
    "georgia (u.s. state)": "ولاية جورجيا",
    "new york (state)": "ولاية نيويورك",
    "washington (state)": "ولاية واشنطن",
    "washington": "واشنطن",
    "washington, d.c.": "واشنطن العاصمة",
    "georgia": "جورجيا",
    "new york": "نيويورك",
    "alabama": "ألاباما",
    "alaska": "ألاسكا",
    "arizona": "أريزونا",
    "arkansas": "أركنساس",
    "california": "كاليفورنيا",
    "colorado": "كولورادو",
    "connecticut": "كونيتيكت",
    "delaware": "ديلاوير",
    "florida": "فلوريدا",
    "hawaii": "هاواي",
    "idaho": "أيداهو",
    "illinois": "إلينوي",
    "indiana": "إنديانا",
    "iowa": "آيوا",
    "kansas": "كانساس",
    "kentucky": "كنتاكي",
    "louisiana": "لويزيانا",
    "maine": "مين",
    "maryland": "ماريلند",
    "massachusetts": "ماساتشوستس",
    "michigan": "ميشيغان",
    "minnesota": "منيسوتا",
    "mississippi": "مسيسيبي",
    "missouri": "ميزوري",
    "montana": "مونتانا",
    "nebraska": "نبراسكا",
    "nevada": "نيفادا",
    "new hampshire": "نيوهامشير",
    "new jersey": "نيوجيرسي",
    "new mexico": "نيومكسيكو",
    "north carolina": "كارولاينا الشمالية",
    "north dakota": "داكوتا الشمالية",
    "ohio": "أوهايو",
    "oklahoma": "أوكلاهوما",
    "oregon": "أوريغن",
    "pennsylvania": "بنسلفانيا",
    "rhode island": "رود آيلاند",
    "south carolina": "كارولاينا الجنوبية",
    "south dakota": "داكوتا الجنوبية",
    "tennessee": "تينيسي",
    "texas": "تكساس",
    "utah": "يوتا",
    "vermont": "فيرمونت",
    "virginia": "فرجينيا",
    "west virginia": "فرجينيا الغربية",
    "wisconsin": "ويسكونسن",
    "wyoming": "وايومنغ",
}

JAPAN_REGIONAL_LABELS = {
    "gokishichidō": "",
    "saitama": "سايتاما",
    "tohoku": "توهوكو",
    "shikoku": "شيكوكو",
    "kyushu": "كيوشو",
    "kantō": "كانتو",
    "kansai": "كانساي",
    "hokkaido": "هوكايدو",
    "hokuriku": "هوكوريكو",
    "chūgoku": "تشوغوكو",
    "toyama": "توياما",
    "tokushima": "توكوشيما",
    "chiba": "تشيبا",
    "tottori": "توتوري",
    "tochigi": "توتشيغي",
    "iwate": "إيواته",
    "ibaraki": "إيباراكي",
    "ishikawa": "إيشيكاوا",
    "ōsaka": "أوساكا",
    "okayama": "أوكاياما",
    "ehime": "إهيمه",
    "akita": "أكيتا",
    "aomori": "آوموري",
    "aichi": "آيتشي",
    "ōita": "أويتا",
    "okinawa": "أوكيناوا",
    "saga": "ساغا",
    "shimane": "شيمانه",
    "shiga": "شيغا",
    "shizuoka": "شيزوكا",
    "kanagawa": "كاناغاوا",
    "kagoshima": "كاغوشيما",
    "kagawa": "كاغاوا",
    "fukui": "فوكوي",
    "fukuoka": "فوكوكا",
    "fukushima": "فوكوشيما",
    "gifu": "غيفو",
    "gunma": "غونما",
    "kōchi": "كوتشي",
    "kumamoto": "كوماموتو",
    "kyōto": "كيوتو",
    "nagano": "ناغانو",
    "nagasaki": "ناغاساكي",
    "nara": "نارا",
    "mie": "ميه",
    "miyagi": "مياغي",
    "miyazaki": "ميازاكي",
    "yamanashi": "ياماناشي",
    "yamaguchi": "ياماغوتشي",
    "yamagata": "ياماغاتا",
    "wakayama": "واكاياما",
    "hyōgo": "هيوغو",
    "hiroshima prefecture": "عمالة هيروشيما",
    "niigata": "نييغاتا",
    "hokkaidō": "هوكايدو",
}

TURKEY_PROVINCE_LABELS = {
    "adana": "أضنة",
    "adıyaman": "أديامان",
    "afyonkarahisar": "أفيون قره حصار",
    "ağrı": "أغري",
    "aksaray": "آق سراي",
    "amasya": "أماصيا",
    "ankara": "أنقرة",
    "antalya": "أنطاليا",
    "ardahan": "أردهان",
    "artvin": "أرتوين",
    "aydın": "أيدين",
    "balıkesir": "بالق أسير",
    "bartın": "بارتين",
    "batman": "بطمان",
    "bayburt": "بايبورت",
    "bilecik": "بيله جك",
    "bingöl": "بينكل",
    "bitlis": "بتليس",
    "bolu": "بولو",
    "burdur": "بوردور",
    "bursa": "بورصة",
    "çanakkale": "جاناكالي",
    "çankırı": "جانقري",
    "çorum": "جوروم",
    "denizli": "دنيزلي",
    "diyarbakır": "دياربكر",
    "düzce": "دوزجه",
    "edirne": "أدرنة",
    "elazığ": "إلازيغ",
    "erzincan": "أرزينجان",
    "erzurum": "أرضروم",
    "eskişehir": "إسكيشهر",
    "gaziantep": "عنتاب",
    "giresun": "غيرسون",
    "gümüşhane": "كوموش خانة",
    "hakkâri": "حكاري",
    "hatay": "هاتاي",
    "iğdır": "اغدير",
    "isparta": "إسبرطة",
    "istanbul": "إسطنبول",
    "izmir": "إزمير",
    "kahramanmaraş": "قهرمان مرعش",
    "karabük": "كارابوك",
    "karaman": "كارامان",
    "kars": "كارس",
    "kastamonu": "قسطموني",
    "kayseri": "قيصري",
    "kilis": "كلس",
    "kırıkkale": "قيريقكالي",
    "kırklareli": "قرقلر ايلي",
    "kırşehir": "قرشهر",
    "kocaeli": "قوجه ايلي",
    "konya": "قونية",
    "kütahya": "كوتاهية",
    "malatya": "ملطية",
    "manisa": "مانيسا",
    "mardin": "ماردين",
    "mersin": "مرسين",
    "muğla": "موغلا",
    "muş": "موش",
    "nevşehir": "نوشهر",
    "niğde": "نيدا",
    "ordu": "أردو",
    "osmaniye": "عثمانية",
    "rize": "ريزه",
    "sakarya": "صقاريا",
    "samsun": "سامسون",
    "şanlıurfa": "شانلي أورفة",
    "siirt": "سعرد",
    "sinop": "سينوب",
    "sivas": "سيواس",
    "şırnak": "شرناق",
    "tekirdağ": "تكيرداغ",
    "tokat": "توقات",
    "trabzon": "طرابزون",
    "tunceli": "تونجلي",
    "uşak": "أوشاك",
    "van": "وان",
    "yalova": "يالوفا",
    "yozgat": "يوزغات",
    "zonguldak": "زانغولداك",
}


def update_with_lowercased(target: MutableMapping[str, str], mapping: Mapping[str, str]) -> None:
    """Update ``target`` with a lower-cased version of ``mapping``."""

    for key, value in mapping.items():
        if not value:
            continue
        target[key.lower()] = value


def setdefault_with_lowercased(target: MutableMapping[str, str], mapping: Mapping[str, str], name: str = "") -> None:
    """Update ``target`` with a lower-cased version of ``mapping``."""
    added = 0
    for key, value in mapping.items():
        if not value or key.lower() in target:
            continue
        target.setdefault(key.lower(), value)
        added += 1

    logger.debug(f"Added {added} entries to the target mapping, source mapping({name}) {len(mapping)}.")


def _make_japan_labels(data: dict[str, str]) -> dict[str, str]:
    labels_index = {}
    for province_name, province_label in data.items():
        if province_label:
            normalized = province_name.lower()
            labels_index[normalized] = province_label
            labels_index[f"{normalized} prefecture"] = f"محافظة {province_label}"
            labels_index[f"{normalized} region"] = f"منطقة {province_label}"

    return labels_index


def _make_turkey_labels(data: dict[str, str]) -> dict[str, str]:
    labels_index = {}
    for province_name, province_label in data.items():
        if province_label:
            normalized = province_name.lower()
            labels_index[normalized] = province_label
            labels_index[f"{normalized} province"] = f"محافظة {province_label}"
            labels_index[f"districts of {normalized} province"] = f"أقضية محافظة {province_label}"

    return labels_index


COMPANY_LABELS_NEW = {
    "airliner": "طائرات",
    "condiment": "توابل",
    "fraternal service": "خدمات أخوية",
    "health care": "رعاية صحية",
    "internet": "إنترنت",
    "magazine": "مجلات",
    "mass media": "وسائل إعلام",
    "military logistics": "لوجستية عسكرية",
    "rail": "سكك حديدية",
    "submarine": "غواصات",
}

JAPAN_LABELS = _make_japan_labels(JAPAN_REGIONAL_LABELS)
TURKEY_LABELS = _make_turkey_labels(TURKEY_PROVINCE_LABELS)


CITY_LABEL_PATCHES = load_json_mapping("cities/yy2.json")
COUNTRY_LABEL_OVERRIDES = load_json_mapping("geography/P17_2_final_ll.json")
raw_region_overrides = load_json_mapping("geography/popopo.json")


def _handle_the_prefix(label_index: dict[str, str]) -> dict[str, str]:
    """Handle 'the ' prefix in country labels."""
    new_keys = {}
    for key, value in list(label_index.items()):
        if not key.lower().startswith("the ") or not value:
            continue

        trimmed_key = key[len("the ") :].strip()
        if trimmed_key in label_index:
            continue
        new_keys.setdefault(trimmed_key, value)

    logger.debug(f">> _handle_the_prefix() Added {len(new_keys)} entries without 'the ' prefix.")
    return new_keys


def _build_country_label_index() -> dict[str, str]:
    """Return the aggregated translation table for countries and regions."""

    label_index: dict[str, str] = {}

    label_index.update(CITY_TRANSLATIONS_LOWER)  # 10,788

    to_update = {
        "ALL_COUNTRY_AR": all_country_ar,  # 54
        "US_STATES": US_STATES,  # 54
        "COUNTRY_LABEL_OVERRIDES": COUNTRY_LABEL_OVERRIDES,  # 1778
        "COUNTRY_ADMIN_LABELS": COUNTRY_ADMIN_LABELS,  # 1782
        "MAIN_REGION_TRANSLATIONS": MAIN_REGION_TRANSLATIONS,  # 823
        "raw_region_overrides": raw_region_overrides,  # 1782
        "SECONDARY_REGION_TRANSLATIONS": SECONDARY_REGION_TRANSLATIONS,  # 176
        "INDIA_REGION_TRANSLATIONS": INDIA_REGION_TRANSLATIONS,  # 1424
        # "CITY_LABEL_PATCHES": CITY_LABEL_PATCHES,                          # 5191
        # "pf_keys2": pf_keys2,                                              # 35730,
        # "US_COUNTY_TRANSLATIONS": US_COUNTY_TRANSLATIONS,                  # 2998
        # "JAPAN_LABELS": JAPAN_LABELS,                                      # 162
        # "TURKEY_LABELS": TURKEY_LABELS,                                    # 243
        # "COMPANY_LABELS_NEW": COMPANY_LABELS_NEW,                          # 10
    }
    for name, mapping in to_update.items():
        logger.debug(f">> _build_country_label_index() Updating labels for {name}, entries: {len(mapping)}")
        update_with_lowercased(label_index, mapping)

    label_index.update(  # Specific overrides used by downstream consumers.
        {
            "indycar": "أندي كار",
            "indiana": "إنديانا",
            "motorsport": "رياضة محركات",
            "indianapolis": "إنديانابوليس",
            "sports in indiana": "الرياضة في إنديانا",
            "igbo": "إغبو",
        }
    )
    no_prefix = _handle_the_prefix(label_index)  # 276
    label_index.update(no_prefix)

    setdefault_with_lowercased(label_index, TAXON_TABLE, "TAXON_TABLE")  # 5324

    setdefault_with_lowercased(label_index, BASE_POP_FINAL_5, "BASE_POP_FINAL_5")  # 124

    return label_index


NEW_P17_FINAL = _build_country_label_index()  # 68,981


def get_from_new_p17_aliases(text: str, default: str | None = "") -> str:
    """Look up the Arabic label for a term in alias mappings."""
    result = (
        COMPANY_LABELS_NEW.get(text)
        or TURKEY_LABELS.get(text)
        or JAPAN_LABELS.get(text)
        or US_COUNTY_TRANSLATIONS.get(text)
        or pf_keys2.get(text)
        or CITY_LABEL_PATCHES.get(text)
    )
    return result or default


def get_from_new_p17_final(text: str, default: str | None = "") -> str:
    """Look up the Arabic label for a term in the ``NEW_P17_FINAL`` mapping."""

    lower_text = text.lower()
    # result = NEW_P17_FINAL.get(lower_text) or get_from_new_p17_aliases(lower_text)
    result = get_from_new_p17_aliases(lower_text) or NEW_P17_FINAL.get(lower_text)

    return result or default


__all__ = [
    "COUNTRY_LABEL_OVERRIDES",
    "get_from_new_p17_final",
]

len_print.data_len(
    "labels_country.py",
    {
        "COUNTRY_LABEL_OVERRIDES": COUNTRY_LABEL_OVERRIDES,
        "CITY_LABEL_PATCHES": CITY_LABEL_PATCHES,  # 5,191
        "NEW_P17_FINAL": NEW_P17_FINAL,
    },
)



#
import pytest
from load_one_data import dump_diff, dump_same_and_not_same, one_dump_test

from ArWikiCats import resolve_label_ar
from ArWikiCats.new.resolve_films_bots.film_keys_bot import Films

data_1 = {
    "3d comics": "قصص مصورة ثلاثية الأبعاد",
    "3d film series": "سلاسل أفلام ثلاثية الأبعاد",
    "3d soap opera": "مسلسلات طويلة ثلاثية الأبعاد",
    "3d television episodes": "حلقات تلفزيونية ثلاثية الأبعاد",
    "3d television films": "أفلام تلفزيونية ثلاثية الأبعاد",
    "3d television miniseries": "مسلسلات قصيرة تلفزيونية ثلاثية الأبعاد",
    "3d television news": "أخبار تلفزيونية ثلاثية الأبعاد",
    "3d television programmes": "برامج تلفزيونية ثلاثية الأبعاد",
    "3d television programs": "برامج تلفزيونية ثلاثية الأبعاد",
    "3d television series": "مسلسلات تلفزيونية ثلاثية الأبعاد",
    "3d video games": "ألعاب فيديو ثلاثية الأبعاد",
    "3d web series": "مسلسلات ويب ثلاثية الأبعاد",
    "4d comics": "قصص مصورة رباعية الأبعاد",
    "4d film series": "سلاسل أفلام رباعية الأبعاد",
    "4d soap opera": "مسلسلات طويلة رباعية الأبعاد",
    "4d television episodes": "حلقات تلفزيونية رباعية الأبعاد",
    "4d television films": "أفلام تلفزيونية رباعية الأبعاد",
    "4d television miniseries": "مسلسلات قصيرة تلفزيونية رباعية الأبعاد",
    "4d television news": "أخبار تلفزيونية رباعية الأبعاد",
    "4d television programmes": "برامج تلفزيونية رباعية الأبعاد",
    "4d television programs": "برامج تلفزيونية رباعية الأبعاد",
    "4d television series": "مسلسلات تلفزيونية رباعية الأبعاد",
    "4d video games": "ألعاب فيديو رباعية الأبعاد",
    "4d web series": "مسلسلات ويب رباعية الأبعاد",
    "action comedy comics": "قصص مصورة حركة كوميدية",
    "action comedy film series": "سلاسل أفلام حركة كوميدية",
    "action comedy soap opera": "مسلسلات طويلة حركة كوميدية",
    "action comedy television episodes": "حلقات تلفزيونية حركة كوميدية",
    "action comedy television films": "أفلام تلفزيونية حركة كوميدية",
    "action comedy television miniseries": "مسلسلات قصيرة تلفزيونية حركة كوميدية",
    "action comedy television news": "أخبار تلفزيونية حركة كوميدية",
    "action comedy television programmes": "برامج تلفزيونية حركة كوميدية",
    "action comedy television programs": "برامج تلفزيونية حركة كوميدية",
    "action comedy television series": "مسلسلات تلفزيونية حركة كوميدية",
    "action comedy video games": "ألعاب فيديو حركة كوميدية",
    "action comedy web series": "مسلسلات ويب حركة كوميدية",
    "action comics": "قصص مصورة حركة",
    "action film series": "سلاسل أفلام حركة",
    "action soap opera": "مسلسلات طويلة حركة",
    "action television episodes": "حلقات تلفزيونية حركة",
    "action television films": "أفلام تلفزيونية حركة",
    "action television miniseries": "مسلسلات قصيرة تلفزيونية حركة",
    "action television news": "أخبار تلفزيونية حركة",
    "action television programmes": "برامج تلفزيونية حركة",
    "action television programs": "برامج تلفزيونية حركة",
    "action television series": "مسلسلات تلفزيونية حركة",
    "action thriller comics": "قصص مصورة إثارة حركة",
    "action thriller film series": "سلاسل أفلام إثارة حركة",
    "action thriller soap opera": "مسلسلات طويلة إثارة حركة",
}

data_2 = {
    "yemeni children's-animated-adventure-television films": "أفلام مغامرات رسوم متحركة تلفزيونية يمنية للأطفال",
    "children's-animated-adventure-television films": "أفلام مغامرات رسوم متحركة تلفزيونية للأطفال",
    "action television characters": "شخصيات تلفزيونية حركة",
}


@pytest.mark.parametrize("category, expected", data_1.items(), ids=data_1.keys())
@pytest.mark.fast
def test_films_keys2_batch_1(category: str, expected: str) -> None:
    label = Films(category)
    assert label == expected


TEMPORAL_CASES = [
    ("test_films_keys2_batch_1", data_1, Films),
    # ("test_films_keys2_batch_2", data_1, resolve_label_ar),
]


@pytest.mark.parametrize("name,data,callback", TEMPORAL_CASES)
@pytest.mark.dump
def test_all_dump(name: str, data: dict[str, str], callback: callable) -> None:
    expected, diff_result = one_dump_test(data, callback)

    dump_diff(diff_result, name)
    dump_same_and_not_same(data, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



#!/usr/bin/python3
"""
"""

jobs_formatted_data_shared = {
    "{en_sport} competitions": "منافسات {sport_jobs}",
    "domestic {en_sport}": "{sport_jobs} محلية",
    "professional {en_sport}": "{sport_jobs} للمحترفين",
    "indoor {en_sport}": "{sport_jobs} داخل الصالات",
    "defunct indoor {en_sport}": "{sport_jobs} داخل الصالات سابقة",
    "{en_sport} cup": "كؤوس {sport_jobs}",
    "{en_sport} cups": "كؤوس {sport_jobs}",
    "defunct outdoor {en_sport} cups": "كؤوس {sport_jobs} في الهواء الطلق سابقة",
    "defunct {en_sport} cup": "كؤوس {sport_jobs} سابقة",
    "defunct {en_sport} cups": "كؤوس {sport_jobs} سابقة",
    "domestic womens {en_sport} cups": "كؤوس {sport_jobs} محلية للسيدات",
    "domestic {en_sport} cup": "كؤوس {sport_jobs} محلية",
    "domestic {en_sport} cups": "كؤوس {sport_jobs} محلية",
}

jobs_formatted_data = {
    # "{en_sport}": "{sport_jobs}",
    "domestic womens {en_sport} leagues": "دوريات {sport_jobs} محلية للسيدات",
    "domestic womens {en_sport}": "{sport_jobs} محلية للسيدات",
    "domestic {en_sport} leagues": "دوريات {sport_jobs} محلية",
    "defunct indoor {en_sport} cups": "كؤوس {sport_jobs} داخل الصالات سابقة",
    "indoor {en_sport} cups": "كؤوس {sport_jobs} داخل الصالات",
    "indoor {en_sport} leagues": "دوريات {sport_jobs} داخل الصالات",
    "defunct indoor {en_sport} leagues": "دوريات {sport_jobs} داخل الصالات سابقة",
    "outdoor {en_sport} cups": "كؤوس {sport_jobs} في الهواء الطلق",
    "professional {en_sport} cups": "كؤوس {sport_jobs} للمحترفين",
    "professional {en_sport} leagues": "دوريات {sport_jobs} للمحترفين",
    "under-13 {en_sport}": "{sport_jobs} تحت 13 سنة",
    "under-14 {en_sport}": "{sport_jobs} تحت 14 سنة",
    "under-15 {en_sport}": "{sport_jobs} تحت 15 سنة",
    "under-16 {en_sport}": "{sport_jobs} تحت 16 سنة",
    "under-17 {en_sport}": "{sport_jobs} تحت 17 سنة",
    "under-18 {en_sport}": "{sport_jobs} تحت 18 سنة",
    "under-19 {en_sport}": "{sport_jobs} تحت 19 سنة",
    "under-20 {en_sport}": "{sport_jobs} تحت 20 سنة",
    "under-21 {en_sport}": "{sport_jobs} تحت 21 سنة",
    "under-23 {en_sport}": "{sport_jobs} تحت 23 سنة",
    "under-24 {en_sport}": "{sport_jobs} تحت 24 سنة",
    "amateur {en_sport} championships": "بطولات {sport_jobs} للهواة",
    "amateur {en_sport}": "{sport_jobs} للهواة",
    "college {en_sport}": "{sport_jobs} الكليات",
    "current {en_sport} seasons": "مواسم {sport_jobs} حالية",
    "defunct outdoor {en_sport} leagues": "دوريات {sport_jobs} في الهواء الطلق سابقة",
    "defunct outdoor {en_sport}": "{sport_jobs} في الهواء الطلق سابقة",
    "defunct {en_sport} leagues": "دوريات {sport_jobs} سابقة",
    "defunct {en_sport} teams": "فرق {sport_jobs} سابقة",
    "defunct {en_sport}": "{sport_jobs} سابقة",
    "fictional {en_sport}": "{sport_jobs} خيالية",
    "fifth level {en_sport} league": "دوريات {sport_jobs} من الدرجة الخامسة",
    "fifth level {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الخامسة",
    "fifth tier {en_sport} league": "دوريات {sport_jobs} من الدرجة الخامسة",
    "fifth tier {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الخامسة",
    "first level {en_sport} league": "دوريات {sport_jobs} من الدرجة الأولى",
    "first level {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الأولى",
    "first tier {en_sport} league": "دوريات {sport_jobs} من الدرجة الأولى",
    "first tier {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الأولى",
    "first-class {en_sport}": "{sport_jobs} من الدرجة الأولى",
    "first-class {en_sport} teams": "فرق {sport_jobs} من الدرجة الأولى",
    "fourth level {en_sport} league": "دوريات {sport_jobs} من الدرجة الرابعة",
    "fourth level {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الرابعة",
    "fourth tier {en_sport} league": "دوريات {sport_jobs} من الدرجة الرابعة",
    "fourth tier {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الرابعة",
    "grand slam ({en_sport}) tournament champions": "أبطال بطولات {sport_jobs} كبرى",
    "grand slam ({en_sport}) tournaments": "بطولات {sport_jobs} كبرى",
    "grand slam ({en_sport})": "بطولات {sport_jobs} كبرى",
    "international mens {en_sport} players": "لاعبو {sport_jobs} دوليون",
    "international mens {en_sport} playerss": "لاعبو {sport_jobs} دوليون",
    "international mens {en_sport}": "{sport_jobs} دولية للرجال",
    "international womens {en_sport} players": "لاعبات {sport_jobs} دوليات",
    "international womens {en_sport} playerss": "لاعبات {sport_jobs} دوليات",
    "international womens {en_sport}": "{sport_jobs} دولية للسيدات",
    "International {en_sport} competition": "منافسات {sport_jobs} دولية",
    "International {en_sport} competitions": "منافسات {sport_jobs} دولية",
    "international {en_sport} managers": "مدربو {sport_jobs} دوليون",
    "international {en_sport} players": "لاعبو {sport_jobs} دوليون",
    "international {en_sport} playerss": "لاعبو {sport_jobs} دوليون",
    "International {en_sport} races": "سباقات {sport_jobs} دولية",
    "International {en_sport}": "{sport_jobs} دولية",
    "international youth {en_sport}": "{sport_jobs} شبابية دولية",
    "mens international {en_sport} players": "لاعبو {sport_jobs} دوليون",
    "mens international {en_sport} playerss": "لاعبو {sport_jobs} دوليون",
    "mens international {en_sport}": "{sport_jobs} دولية للرجال",
    "mens {en_sport} teams": "فرق {sport_jobs} رجالية",
    "mens {en_sport}": "{sport_jobs} رجالية",
    "military {en_sport}": "{sport_jobs} عسكرية",
    "multi-national {en_sport} championships": "بطولات {sport_jobs} متعددة الجنسيات",
    "multi-national {en_sport} league": "دوريات {sport_jobs} متعددة الجنسيات",
    "multi-national {en_sport} leagues": "دوريات {sport_jobs} متعددة الجنسيات",
    "national a {en_sport} teams": "منتخبات {sport_jobs} للمحليين",
    "national a. {en_sport} teams": "منتخبات {sport_jobs} للمحليين",
    "national b {en_sport} teams": "منتخبات {sport_jobs} رديفة",
    "national b. {en_sport} teams": "منتخبات {sport_jobs} رديفة",
    "national junior mens {en_sport} teams": "منتخبات {sport_jobs} وطنية للناشئين",
    "national junior {en_sport} teams": "منتخبات {sport_jobs} وطنية للناشئين",
    "national mens {en_sport} teams": "منتخبات {sport_jobs} وطنية رجالية",
    "national mens {en_sport}": "منتخبات {sport_jobs} وطنية للرجال",
    "national reserve {en_sport} teams": "منتخبات {sport_jobs} وطنية احتياطية",
    "national under-13 {en_sport}": "منتخبات {sport_jobs} تحت 13 سنة",
    "national under-14 {en_sport}": "منتخبات {sport_jobs} تحت 14 سنة",
    "national under-15 {en_sport}": "منتخبات {sport_jobs} تحت 15 سنة",
    "national under-16 {en_sport}": "منتخبات {sport_jobs} تحت 16 سنة",
    "national under-17 {en_sport}": "منتخبات {sport_jobs} تحت 17 سنة",
    "national under-18 {en_sport}": "منتخبات {sport_jobs} تحت 18 سنة",
    "national under-19 {en_sport}": "منتخبات {sport_jobs} تحت 19 سنة",
    "national under-20 {en_sport}": "منتخبات {sport_jobs} تحت 20 سنة",
    "national under-21 {en_sport}": "منتخبات {sport_jobs} تحت 21 سنة",
    "national under-23 {en_sport}": "منتخبات {sport_jobs} تحت 23 سنة",
    "national under-24 {en_sport}": "منتخبات {sport_jobs} تحت 24 سنة",
    "national womens {en_sport} teams": "منتخبات {sport_jobs} وطنية نسائية",
    "national womens {en_sport}": "منتخبات {sport_jobs} وطنية للسيدات",
    "national {en_sport} champions": "أبطال بطولات {sport_jobs} وطنية",
    "national {en_sport} championships": "بطولات {sport_jobs} وطنية",
    "national {en_sport} league": "دوريات {sport_jobs} وطنية",
    "national {en_sport} leagues": "دوريات {sport_jobs} وطنية",
    "national {en_sport} team results": "نتائج منتخبات {sport_jobs} وطنية",
    "national {en_sport} teams": "منتخبات {sport_jobs} وطنية",
    "national {en_sport}": "منتخبات {sport_jobs} وطنية",
    "national youth {en_sport} teams": "منتخبات {sport_jobs} وطنية شبابية",
    "outdoor {en_sport} leagues": "دوريات {sport_jobs} في الهواء الطلق",
    "outdoor {en_sport}": "{sport_jobs} في الهواء الطلق",
    "premier {en_sport} league": "دوريات {sport_jobs} من الدرجة الممتازة",
    "premier {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الممتازة",
    "reserve {en_sport} teams": "فرق {sport_jobs} احتياطية",
    "second level {en_sport} league": "دوريات {sport_jobs} من الدرجة الثانية",
    "second level {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الثانية",
    "second tier {en_sport} league": "دوريات {sport_jobs} من الدرجة الثانية",
    "second tier {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الثانية",
    "seventh level {en_sport} league": "دوريات {sport_jobs} من الدرجة السابعة",
    "seventh level {en_sport} leagues": "دوريات {sport_jobs} من الدرجة السابعة",
    "seventh tier {en_sport} league": "دوريات {sport_jobs} من الدرجة السابعة",
    "seventh tier {en_sport} leagues": "دوريات {sport_jobs} من الدرجة السابعة",
    "sixth level {en_sport} league": "دوريات {sport_jobs} من الدرجة السادسة",
    "sixth level {en_sport} leagues": "دوريات {sport_jobs} من الدرجة السادسة",
    "sixth tier {en_sport} league": "دوريات {sport_jobs} من الدرجة السادسة",
    "sixth tier {en_sport} leagues": "دوريات {sport_jobs} من الدرجة السادسة",
    "third level {en_sport} league": "دوريات {sport_jobs} من الدرجة الثالثة",
    "third level {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الثالثة",
    "third tier {en_sport} league": "دوريات {sport_jobs} من الدرجة الثالثة",
    "third tier {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الثالثة",
    "top level {en_sport} league": "دوريات {sport_jobs} من الدرجة الأولى",
    "top level {en_sport} leagues": "دوريات {sport_jobs} من الدرجة الأولى",
    "womens international {en_sport} players": "لاعبات {sport_jobs} دوليات",
    "womens international {en_sport} playerss": "لاعبات {sport_jobs} دوليات",
    "womens international {en_sport}": "{sport_jobs} دولية للسيدات",
    "womens {en_sport} teams": "فرق {sport_jobs} نسائية",
    "womens {en_sport}": "{sport_jobs} نسائية",
    "{en_sport} chairmen and investors": "رؤساء ومسيرو {sport_jobs}",
    "{en_sport} league teams": "فرق دوري {sport_jobs}",
    "{en_sport} league": "دوري {sport_jobs}",
    "{en_sport} leagues": "دوريات {sport_jobs}",
    "{en_sport} olympic bronze medalists": "ميداليات {sport_jobs} برونزية أولمبية",
    "{en_sport} olympic gold medalists": "ميداليات {sport_jobs} ذهبية أولمبية",
    "{en_sport} olympic silver medalists": "ميداليات {sport_jobs} فضية أولمبية",
    "{en_sport} races": "سباقات {sport_jobs}",
    "{en_sport} super leagues": "دوريات سوبر {sport_jobs}",
    "youth international {en_sport}": "{sport_jobs} دولية شبابية",
    "youth {en_sport}": "{sport_jobs} شبابية",
    # Category:Multi-national women's basketball leagues in Europe
    "multi-national womens {en_sport} leagues": "دوريات {sport_jobs} نسائية متعددة الجنسيات",
    # Category:National junior women's goalball teams
    "national junior womens {en_sport} teams": "منتخبات {sport_jobs} للناشئات",
}

jobs_formatted_data.update(jobs_formatted_data_shared)

# NOTE: used in countries_names_sport_multi_v2.py and nats_and_sports.py
labels_formatted_data = {
    "{en_sport}": "{sport_label}",
    "{en_sport} finals": "نهائيات {sport_label}",
    "olympic gold medalists in {en_sport}": "فائزون بميداليات ذهبية أولمبية في {sport_label}",
    "olympic silver medalists in {en_sport}": "فائزون بميداليات فضية أولمبية في {sport_label}",
    "olympic bronze medalists in {en_sport}": "فائزون بميداليات برونزية أولمبية في {sport_label}",
    "{en_sport} league": "دوري {sport_label}",
    "{en_sport} champions": "أبطال {sport_label}",
    "olympics {en_sport}": "{sport_label} في الألعاب الأولمبية",
    "summer olympics {en_sport}": "{sport_label} في الألعاب الأولمبية الصيفية",
    "winter olympics {en_sport}": "{sport_label} في الألعاب الأولمبية الشتوية",
}


teams_formatted_data = {
    "amateur {en_sport} world cup": "كأس العالم {sport_team} للهواة",
    "mens {en_sport} world cup": "كأس العالم {sport_team} للرجال",
    "womens {en_sport} world cup": "كأس العالم {sport_team} للسيدات",
    "{en_sport} world cup": "كأس العالم {sport_team}",
    "youth {en_sport} world cup": "كأس العالم {sport_team} للشباب",
    "international {en_sport} council": "المجلس الدولي {sport_team}",
    "mens {en_sport} championship": "بطولة {sport_team} للرجال",
    "mens {en_sport} world championship": "بطولة العالم {sport_team} للرجال",
    "outdoor world {en_sport} championship": "بطولة العالم {sport_team} في الهواء الطلق",
    "womens world {en_sport} championship": "بطولة العالم {sport_team} للسيدات",
    "womens {en_sport} championship": "بطولة {sport_team} للسيدات",
    "womens {en_sport} world championship": "بطولة العالم {sport_team} للسيدات",
    "world amateur {en_sport} championship": "بطولة العالم {sport_team} للهواة",
    "world champion national {en_sport} teams": "أبطال بطولة العالم {sport_team}",
    "world junior {en_sport} championship": "بطولة العالم {sport_team} للناشئين",
    "world outdoor {en_sport} championship": "بطولة العالم {sport_team} في الهواء الطلق",
    "world wheelchair {en_sport} championship": "بطولة العالم {sport_team} على الكراسي المتحركة",
    "world {en_sport} amateur championship": "بطولة العالم {sport_team} للهواة",
    "world {en_sport} championship": "بطولة العالم {sport_team}",
    "world {en_sport} championship competitors": "منافسو بطولة العالم {sport_team}",
    "world {en_sport} championship medalists": "فائزون بميداليات بطولة العالم {sport_team}",
    "world {en_sport} junior championship": "بطولة العالم {sport_team} للناشئين",
    "world {en_sport} youth championship": "بطولة العالم {sport_team} للشباب",
    "world youth {en_sport} championship": "بطولة العالم {sport_team} للشباب",
    "{en_sport} amateur world championship": "بطولة العالم {sport_team} للهواة",
    "{en_sport} junior world championship": "بطولة العالم {sport_team} للناشئين",
    "{en_sport} world amateur championship": "بطولة العالم {sport_team} للهواة",
    "{en_sport} world championship": "بطولة العالم {sport_team}",
    "{en_sport} world junior championship": "بطولة العالم {sport_team} للناشئين",
    "{en_sport} world youth championship": "بطولة العالم {sport_team} للشباب",
    "{en_sport} youth world championship": "بطولة العالم {sport_team} للشباب",
}


__all__ = [
    "jobs_formatted_data_shared",
    "jobs_formatted_data",
    "labels_formatted_data",
    "teams_formatted_data",
]



import pytest
from load_one_data import dump_diff, dump_same_and_not_same, one_dump_test

from ArWikiCats import resolve_label_ar

data0 = {
    "15th-century Buddhist temples in China": "معابد بوذية في الصين القرن 15",
    "18th-century Buddhist temples in China": "معابد بوذية في الصين القرن 18",
    "18th-century Buddhist temples in Thailand": "معابد بوذية في تايلاند القرن 18",
    "1450s murders in ireland": "",
    "21st century executions in kentucky": "",
    "6th century kings of italy": "",
}

data1 = {
    "Lists of Buddhist buildings and structures": "قوائم مبان ومنشآت بوذية",
    "Lists of Hindu buildings and structures": "قوائم مبان ومنشآت هندوسية",
    "Lists of Islamic buildings and structures": "قوائم مبان ومنشآت إسلامية",
    "Lists of Jewish buildings and structures": "قوائم مبان ومنشآت يهودية",
    "lists of religious buildings and structures": "قوائم مبان ومنشآت دينية",
    "2000s films": "أفلام إنتاج عقد 2000",
    "1650s films": "أفلام إنتاج عقد 1650",
    "1st-millennium literature": "أدب الألفية 1",
    "20th-century Ghanaian literature": "أدب غاني في القرن 20",
    "20th-century Mexican literature": "أدب مكسيكي في القرن 20",
    "20th-century Taiwanese literature": "أدب تايواني في القرن 20",
    "20th-century Ukrainian literature": "أدب أوكراني في القرن 20",
    "20th-century Zimbabwean literature": "أدب زيمبابوي في القرن 20",
    "21st-century Ghanaian literature": "أدب غاني في القرن 21",
    "21st-century Moroccan literature": "أدب مغربي في القرن 21",
    "21st-century Taiwanese literature": "أدب تايواني في القرن 21",
    "21st-century Ukrainian literature": "أدب أوكراني في القرن 21",
    "21st-century Zimbabwean literature": "أدب زيمبابوي في القرن 21",
    "10th-century Buddhist temples": "معابد بوذية القرن 10",
    "10th-century Hindu temples": "معابد هندوسية القرن 10",
    "11th-century Buddhist temples": "معابد بوذية القرن 11",
    "11th-century Hindu temples": "معابد هندوسية القرن 11",
    "12th-century Buddhist temples": "معابد بوذية القرن 12",
    "12th-century Hindu temples": "معابد هندوسية القرن 12",
    "13th-century Buddhist temples": "معابد بوذية القرن 13",
    "13th-century Hindu temples": "معابد هندوسية القرن 13",
    "14th-century Buddhist temples": "معابد بوذية القرن 14",
    "14th-century Hindu temples": "معابد هندوسية القرن 14",
    "15th-century Buddhist temples": "معابد بوذية القرن 15",
    "15th-century Hindu temples": "معابد هندوسية القرن 15",
    "16th-century Buddhist temples": "معابد بوذية القرن 16",
    "16th-century Hindu temples": "معابد هندوسية القرن 16",
    "17th-century Buddhist temples": "معابد بوذية القرن 17",
    "17th-century Hindu temples": "معابد هندوسية القرن 17",
    "18th-century Buddhist temples": "معابد بوذية القرن 18",
    "18th-century Hindu temples": "معابد هندوسية القرن 18",
    "19th-century Buddhist temples": "معابد بوذية القرن 19",
    "19th-century Hindu temples": "معابد هندوسية القرن 19",
    "1st-century BC Hindu temples": "معابد هندوسية القرن 1 ق م",
    "1st-century Hindu temples": "معابد هندوسية القرن 1",
    "20th-century Buddhist temples": "معابد بوذية القرن 20",
    "20th-century Hindu temples": "معابد هندوسية القرن 20",
    "20th-century Jain temples": "معابد جاينية القرن 20",
    "21st-century Buddhist temples": "معابد بوذية القرن 21",
    "21st-century Hindu temples": "معابد هندوسية القرن 21",
    "2nd-century BC Hindu temples": "معابد هندوسية القرن 2 ق م",
    "2nd-century Hindu temples": "معابد هندوسية القرن 2",
    "3rd-century BC Hindu temples": "معابد هندوسية القرن 3 ق م",
    "3rd-century Buddhist temples": "معابد بوذية القرن 3",
    "4th-century Buddhist temples": "معابد بوذية القرن 4",
    "4th-century Hindu temples": "معابد هندوسية القرن 4",
    "5th-century BC Hindu temples": "معابد هندوسية القرن 5 ق م",
    "5th-century Buddhist temples": "معابد بوذية القرن 5",
    "5th-century Hindu temples": "معابد هندوسية القرن 5",
    "6th-century Buddhist temples": "معابد بوذية القرن 6",
    "6th-century Hindu temples": "معابد هندوسية القرن 6",
    "7th-century Buddhist temples": "معابد بوذية القرن 7",
    "7th-century Hindu temples": "معابد هندوسية القرن 7",
    "8th-century Buddhist temples": "معابد بوذية القرن 8",
    "8th-century Hindu temples": "معابد هندوسية القرن 8",
    "9th-century Buddhist temples": "معابد بوذية القرن 9",
    "9th-century Hindu temples": "معابد هندوسية القرن 9",
    "1610s operas": "أوبيرات عقد 1610",
    "1620s operas": "أوبيرات عقد 1620",
    "1630s operas": "أوبيرات عقد 1630",
    "1640s operas": "أوبيرات عقد 1640",
    "1650s operas": "أوبيرات عقد 1650",
    "1670s operas": "أوبيرات عقد 1670",
    "1680s operas": "أوبيرات عقد 1680",
    "1690s operas": "أوبيرات عقد 1690",
    "1680s essays": "مقالات عقد 1680",
    "1690s essays": "مقالات عقد 1690",
    "1610s essays": "مقالات عقد 1610",
    "1640s essays": "مقالات عقد 1640",
    "13th-century Hindu philosophers and theologians": "فلاسفة ولاهوتيون هندوس في القرن 13",
    "16th-century Hindu philosophers and theologians": "فلاسفة ولاهوتيون هندوس في القرن 16",
    "17th-century Hindu philosophers and theologians": "فلاسفة ولاهوتيون هندوس في القرن 17",
    "18th-century Hindu philosophers and theologians": "فلاسفة ولاهوتيون هندوس في القرن 18",
    "19th-century Hindu philosophers and theologians": "فلاسفة ولاهوتيون هندوس في القرن 19",
    "20th-century Hindu philosophers and theologians": "فلاسفة ولاهوتيون هندوس في القرن 20",
    "21st-century Hindu philosophers and theologians": "فلاسفة ولاهوتيون هندوس في القرن 21",
    "1650s television series": "مسلسلات تلفزيونية في عقد 1650",
    "16th-century Anglican bishops in Ireland": "أساقفة أنجليكيون في القرن 16 في أيرلندا",
    "16th-century rabbis in Jerusalem": "حاخامات في القرن 16 في القدس",
    "17th-century Anglican bishops in Ireland": "أساقفة أنجليكيون في القرن 17 في أيرلندا",
    "18th-century Anglican bishops in Ireland": "أساقفة أنجليكيون في القرن 18 في أيرلندا",
    "18th-century Anglican bishops in North America": "أساقفة أنجليكيون في القرن 18 في أمريكا الشمالية",
    "18th-century Anglican bishops in United States": "أساقفة أنجليكيون في القرن 18 في الولايات المتحدة",
    "18th-century people of the Dutch Empire": "أشخاص في القرن 18 في الإمبراطورية الهولندية",
    "18th-century princes of Monaco": "أمراء في القرن 18 في موناكو",
    "18th-century slaves in Morocco": "عبيد في القرن 18 في المغرب",
    "18th-century sultans of Brunei": "سلاطين في القرن 18 في بروناي",
    "1920s in politics by country": "السياسة في عقد 1920 حسب البلد",
    "1950s criminal comedy films": "أفلام كوميديا الجريمة في عقد 1950",
    "1960s black comedy films": "أفلام كوميدية سوداء في عقد 1960",
    "1960s criminal comedy films": "أفلام كوميديا الجريمة في عقد 1960",
    "1970s black comedy films": "أفلام كوميدية سوداء في عقد 1970",
    "1970s criminal comedy films": "أفلام كوميديا الجريمة في عقد 1970",
    "1980s black comedy films": "أفلام كوميدية سوداء في عقد 1980",
    "1980s criminal comedy films": "أفلام كوميديا الجريمة في عقد 1980",
    "19th-century Anglican bishops in Africa": "أساقفة أنجليكيون في القرن 19 في إفريقيا",
    "19th-century Anglican bishops in Asia": "أساقفة أنجليكيون في القرن 19 في آسيا",
    "19th-century Anglican bishops in Australia": "أساقفة أنجليكيون في القرن 19 في أستراليا",
    "19th-century Anglican bishops in Caribbean": "أساقفة أنجليكيون في القرن 19 في الكاريبي",
    "19th-century Anglican bishops in China": "أساقفة أنجليكيون في القرن 19 في الصين",
    "19th-century Anglican bishops in Ireland": "أساقفة أنجليكيون في القرن 19 في أيرلندا",
    "19th-century Anglican bishops in Middle East": "أساقفة أنجليكيون في القرن 19 في الشرق الأوسط",
    "19th-century Anglican bishops in New Zealand": "أساقفة أنجليكيون في القرن 19 في نيوزيلندا",
    "19th-century Anglican bishops in North America": "أساقفة أنجليكيون في القرن 19 في أمريكا الشمالية",
    "19th-century Anglican bishops in Oceania": "أساقفة أنجليكيون في القرن 19 في أوقيانوسيا",
    "19th-century Anglican bishops in Sierra Leone": "أساقفة أنجليكيون في القرن 19 في سيراليون",
    "19th-century Anglican bishops in United States": "أساقفة أنجليكيون في القرن 19 في الولايات المتحدة",
    "19th-century Anglican bishops of Gibraltar": "أساقفة أنجليكيون في القرن 19 في جبل طارق",
    "19th-century princesses of Wales": "أميرات في القرن 19 في ويلز",
    "2000s in politics by country": "السياسة في عقد 2000 حسب البلد",
    "2011 road incidents in Europe": "حوادث طرق في 2011 في أوروبا",
    "2022 local elections in Nigeria": "انتخابات محلية 2022 في نيجيريا",
    "2022–23 EuroLeague by club": "الدوري الأوروبي لكرة السلة 2022–23 حسب النادي",
    "2026 ballot measures in the United States": "إجراءات اقتراع 2026 في الولايات المتحدة",
    "2026 in politics by country": "السياسة في 2026 حسب البلد",
    "2026 state legislature elections in the United States": "انتخابات المجالس التشريعية للولايات 2026 في الولايات المتحدة",
    "20th century women": "نساء في القرن 20",
    "20th-century Anglican bishops in Africa": "أساقفة أنجليكيون في القرن 20 في إفريقيا",
    "20th-century Anglican bishops in Asia": "أساقفة أنجليكيون في القرن 20 في آسيا",
    "20th-century Anglican bishops in Australia": "أساقفة أنجليكيون في القرن 20 في أستراليا",
    "20th-century Anglican bishops in Bermuda": "أساقفة أنجليكيون في القرن 20 في برمودا",
    "20th-century Anglican bishops in Canada": "أساقفة أنجليكيون في القرن 20 في كندا",
    "20th-century Anglican bishops in Caribbean": "أساقفة أنجليكيون في القرن 20 في الكاريبي",
    "20th-century Anglican bishops in China": "أساقفة أنجليكيون في القرن 20 في الصين",
    "20th-century Anglican bishops in Europe": "أساقفة أنجليكيون في القرن 20 في أوروبا",
    "20th-century Anglican bishops in Ghana": "أساقفة أنجليكيون في القرن 20 في غانا",
    "20th-century Anglican bishops in India": "أساقفة أنجليكيون في القرن 20 في الهند",
    "20th-century Anglican bishops in Ireland": "أساقفة أنجليكيون في القرن 20 في أيرلندا",
    "20th-century Anglican bishops in Malawi": "أساقفة أنجليكيون في القرن 20 في ملاوي",
    "20th-century Anglican bishops in Mexico": "أساقفة أنجليكيون في القرن 20 في المكسيك",
    "20th-century Anglican bishops in Middle East": "أساقفة أنجليكيون في القرن 20 في الشرق الأوسط",
    "20th-century Anglican bishops in New Zealand": "أساقفة أنجليكيون في القرن 20 في نيوزيلندا",
    "20th-century Anglican bishops in Nigeria": "أساقفة أنجليكيون في القرن 20 في نيجيريا",
    "20th-century Anglican bishops in North America": "أساقفة أنجليكيون في القرن 20 في أمريكا الشمالية",
    "20th-century Anglican bishops in Oceania": "أساقفة أنجليكيون في القرن 20 في أوقيانوسيا",
    "20th-century Anglican bishops in Sierra Leone": "أساقفة أنجليكيون في القرن 20 في سيراليون",
    "20th-century Anglican bishops in South America": "أساقفة أنجليكيون في القرن 20 في أمريكا الجنوبية",
    "20th-century Anglican bishops in Tanzania": "أساقفة أنجليكيون في القرن 20 في تنزانيا",
    "20th-century Anglican bishops in Uganda": "أساقفة أنجليكيون في القرن 20 في أوغندا",
    "20th-century Anglican bishops in United States": "أساقفة أنجليكيون في القرن 20 في الولايات المتحدة",
    "20th-century Anglican bishops of Gibraltar": "أساقفة أنجليكيون في القرن 20 في جبل طارق",
    "21st-century Anglican bishops in Africa": "أساقفة أنجليكيون في القرن 21 في إفريقيا",
    "21st-century Anglican bishops in Asia": "أساقفة أنجليكيون في القرن 21 في آسيا",
    "21st-century Anglican bishops in Australia": "أساقفة أنجليكيون في القرن 21 في أستراليا",
    "21st-century Anglican bishops in Bermuda": "أساقفة أنجليكيون في القرن 21 في برمودا",
    "21st-century Anglican bishops in Canada": "أساقفة أنجليكيون في القرن 21 في كندا",
    "21st-century Anglican bishops in Caribbean": "أساقفة أنجليكيون في القرن 21 في الكاريبي",
    "21st-century Anglican bishops in China": "أساقفة أنجليكيون في القرن 21 في الصين",
    "21st-century Anglican bishops in Ghana": "أساقفة أنجليكيون في القرن 21 في غانا",
    "21st-century Anglican bishops in India": "أساقفة أنجليكيون في القرن 21 في الهند",
    "21st-century Anglican bishops in Ireland": "أساقفة أنجليكيون في القرن 21 في أيرلندا",
    "21st-century Anglican bishops in Malawi": "أساقفة أنجليكيون في القرن 21 في ملاوي",
    "21st-century Anglican bishops in Mexico": "أساقفة أنجليكيون في القرن 21 في المكسيك",
    "21st-century Anglican bishops in Middle East": "أساقفة أنجليكيون في القرن 21 في الشرق الأوسط",
    "21st-century Anglican bishops in New Zealand": "أساقفة أنجليكيون في القرن 21 في نيوزيلندا",
    "21st-century Anglican bishops in Nigeria": "أساقفة أنجليكيون في القرن 21 في نيجيريا",
    "21st-century Anglican bishops in North America": "أساقفة أنجليكيون في القرن 21 في أمريكا الشمالية",
    "21st-century Anglican bishops in Oceania": "أساقفة أنجليكيون في القرن 21 في أوقيانوسيا",
    "21st-century Anglican bishops in Sierra Leone": "أساقفة أنجليكيون في القرن 21 في سيراليون",
    "21st-century Anglican bishops in South America": "أساقفة أنجليكيون في القرن 21 في أمريكا الجنوبية",
    "21st-century Anglican bishops in Tanzania": "أساقفة أنجليكيون في القرن 21 في تنزانيا",
    "21st-century Anglican bishops in Uganda": "أساقفة أنجليكيون في القرن 21 في أوغندا",
    "21st-century Anglican bishops in United States": "أساقفة أنجليكيون في القرن 21 في الولايات المتحدة",
    "21st-century Anglican bishops of Gibraltar": "أساقفة أنجليكيون في القرن 21 في جبل طارق",
    "21st-century films": "أفلام إنتاج القرن 21",
    "21st-century Methodist bishops in United States": "أساقفة ميثوديون لاهوتيون في القرن 21 في الولايات المتحدة",
    "4th-century bishops in Roman Empire": "أساقفة في القرن 4 في الإمبراطورية الرومانية",
    "8th-century dukes of Spoleto": "دوقات في القرن 8 في سبوليتو",
    "Counties of Appalachia": "مقاطعات في الأبالاش",
    "Counties of England by date of disestablishment": "مقاطعات في إنجلترا حسب تاريخ الانحلال",
    "Counties of Gyeonggi Province": "مقاطعات في مقاطعة غيونغي",
    "Counties of Iran": "مقاطعات في إيران",
    "Members of Parliament for Gers": "أعضاء برلمان في جارس (إقليم فرنسي)",
    "00s establishments in the Roman Empire": "تأسيسات عقد 00 في الإمبراطورية الرومانية",
    "1000s disestablishments in Asia": "انحلالات عقد 1000 في آسيا",
    "13th century establishments in the Roman Empire": "تأسيسات القرن 13 في الإمبراطورية الرومانية",
    "14th-century establishments in India": "تأسيسات القرن 14 في الهند",
    "1990s BC disestablishments in Asia": "انحلالات عقد 1990 ق م في آسيا",
    "1990s disestablishments in Europe": "انحلالات عقد 1990 في أوروبا",
    "19th-century actors by religion": "ممثلون في القرن 19 حسب الدين",
    "19th-century actors": "ممثلون في القرن 19",
    "19th-century people by religion": "أشخاص في القرن 19 حسب الدين",
    "20th-century disestablishments in India": "انحلالات القرن 20 في الهند",
    "April 1983 events in Europe": "أحداث أبريل 1983 في أوروبا",
    "July 2018 events by continent": "أحداث يوليو 2018 حسب القارة",
    "Lists of organizations based in Central America": "قوائم منظمات مقرها في أمريكا الوسطى",
    "1000 disestablishments by country": "انحلالات سنة 1000 حسب البلد",
    "1000 disestablishments in europe": "انحلالات سنة 1000 في أوروبا",
    "awards by country": "جوائز حسب البلد",
    "politics of united states by state": "سياسة الولايات المتحدة حسب الولاية",
    "television series by city of location": "مسلسلات تلفزيونية حسب مدينة الموقع",
    "19th century people": "أشخاص في القرن 19",
    "lists of football players by national team": "قوائم لاعبو كرة قدم حسب المنتخب الوطني",
}


@pytest.mark.parametrize("category, expected", data1.items(), ids=data1.keys())
@pytest.mark.fast
def test_mk3_0(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


to_test = [
    ("test_2_skip2_2", data1),
]


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_peoples(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_label_ar)

    dump_diff(diff_result, name)
    dump_same_and_not_same(data, expected, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



""" """

import re


def get_color_table() -> dict[str, str]:
    """Build a mapping of color names to ANSI templates."""
    # new Define the color codes for different colors
    color_numbers = {
        # 'lightred': 101,
        # 'lightgreen': 102,
        # 'lightpurple': 105,
        # 'lightyellow': 103,
        # 'lightblue': 104,
        # 'lightcyan': 106,
        # 'aqua': 106,
        # 'lightaqua': 107,
        # 'lightwhite': 107,
        # 'lightgray': 107,
        "red": 91,
        "green": 92,
        "yellow": 93,
        "blue": 94,
        "purple": 95,
        "cyan": 96,
        "white": 97,
        "black": 98,
        "grey": 99,
        "gray": 100,
        "underline": 4,
        "invert": 7,
        "blink": 5,
        "lightblack": 108,
        "bold": 1,
    }
    color_table = {x: f"\033[{v}m%s\033[00m" for x, v in color_numbers.items()}

    # Add light versions of the colors to the color table
    for color in ["purple", "yellow", "blue", "red", "green", "cyan", "gray"]:
        color_table[f"light{color}"] = color_table.get(color, 0)

    # Add some additional color names to the color table
    color_table["aqua"] = color_table.get("cyan", 0)
    color_table["lightaqua"] = color_table.get("cyan", 0)
    color_table["lightgrey"] = color_table.get("gray", 0)
    color_table["grey"] = color_table.get("gray", 0)
    color_table["lightwhite"] = color_table.get("gray", 0)
    color_table["light"] = 0

    return color_table


color_table = get_color_table()


def make_str(textm: str) -> str:
    """
    Prints the given text with color formatting.

    The text can contain color tags like '<<color>>' where 'color' is the name of the color.
    The color will be applied to the text that follows the tag, until the end of the string or until a '<<default>>' tag is found.

    :param textm: The text to print. Can contain color tags.
    """
    # Define a pattern for color tags
    _color_pat = r"((:?\w+|previous);?(:?\w+|previous)?)"
    # Compile a regex for color tags
    colorTagR = re.compile(rf"(?:\03{{|<<){_color_pat}(?:}}|>>)")

    # Initialize a stack for color tags
    color_stack = ["default"]

    # If the input is not a string, print it as is and return
    if not isinstance(textm, str):
        return textm

    # If the text does not contain any color tags, print it as is and return
    if "\03" not in textm and "<<" not in textm:
        return textm

    # Split the text into parts based on the color tags
    text_parts = colorTagR.split(textm) + ["default"]

    # Enumerate the parts for processing
    enu = enumerate(zip(text_parts[::4], text_parts[1::4]))

    # Initialize the string to be printed
    toprint = ""

    # Process each part of the text
    for _, (text, next_color) in enu:
        # Get the current color from the color stack
        # print(f"i: {index}, {text=}, {next_color=}")
        current_color = color_stack[-1]

        # If the next color is 'previous', pop the color stack to get the previous color
        if next_color == "previous":
            if len(color_stack) > 1:  # keep the last element in the stack
                color_stack.pop()
            next_color = color_stack[-1]
        else:
            # If the next color is not 'previous', add it to the color stack
            color_stack.append(next_color)

        # Get the color code for the current color
        cc = color_table.get(current_color, "")

        # If the color code is not empty, apply it to the text
        if cc:
            text = cc % text

        # Add the colored text to the string to be printed
        toprint += text

    # Print the final colored text
    return toprint


__all__ = [
    "make_str",
]



#!/usr/bin/python3
"""
Nationality system with full refactoring and full type hints.
All comments are in English only.
"""

from __future__ import annotations

from typing import Any, Dict, List, Tuple, TypedDict

from ...helps import len_print
from ..utils.json_dir import open_json_file

regex_line = r"""
"male": "([^"]+)?",
\s+"males": "([^"]+)?",
\s+"female": "([^"]+)?",
\s+"females": "([^"]+)?",
\s+"the_male": "([^"]+)?",
\s+"the_female": "([^"]+)?",
\s+"en": "([^"]+)?",
\s+"ar": "([^"]+)?"

"male": "$1",
"males": "$2",
"female": "$3",
"females": "$4",
"the_male": "$5",
"the_female": "$6",
"en": "$7",
"ar": "$8"
"""
# =====================================================================
# Type aliases
# =====================================================================

countries_en_as_nationality_keys = [
    "antigua and barbuda",
    "botswana",
    "central african republic",
    "chinese taipei",
    "democratic republic of congo",
    "democratic-republic-of-congo",
    "dominican republic",
    "federated states of micronesia",
    "federated states-of micronesia",
    "georgia (country)",
    "hong kong",
    "ireland",
    "kiribati",
    "kyrgyz",
    "lesotho",
    "liechtenstein",
    "new zealand",
    "northern ireland",
    "republic of congo",
    "republic of ireland",
    "republic-of ireland",
    "republic-of-congo",
    "são toméan",
    "trinidad and tobago",
    "turkmen",
    "turkmenistan",
    "uzbek",
    "vatican",
    "west india",
]


class NationalityEntry(TypedDict):
    """Represents one nationality block with all fields always present as strings."""

    male: str
    males: str
    female: str
    females: str
    the_male: str
    the_female: str
    en: str
    ar: str


AllNatDict = Dict[str, NationalityEntry]
LookupTable = Dict[str, str]


def build_nationality_structure(val):
    return {
        "male": val.get("male", ""),
        "males": val.get("males", ""),
        "female": val.get("female", ""),
        "females": val.get("females", ""),
        "the_female": val.get("the_female", ""),
        "the_male": val.get("the_male", ""),
        "en": val.get("en", ""),
        "ar": val.get("ar", ""),
    }


# =====================================================================
# Section 1: Load and prepare JSON sources
# =====================================================================


raw_sub_nat_additional_to_check = {
    "muslims": {
        "male": "مسلم",
        "males": "مسلمون",
        "female": "مسلمة",
        "females": "مسلمات",
        "the_male": "المسلم",
        "the_female": "المسلمة",
        "en": "",
        "ar": "الإسلام",
    },
    "muslim": {
        "male": "مسلم",
        "males": "مسلمون",
        "female": "مسلمة",
        "females": "مسلمات",
        "the_male": "المسلم",
        "the_female": "المسلمة",
        "en": "muslims",
        "ar": "الإسلام",
    },
}

raw_sub_nat_additional = {
    "jews": {
        "male": "يهودي",
        "males": "يهود",
        "female": "يهودية",
        "females": "يهوديات",
        "the_male": "اليهودي",
        "the_female": "اليهودية",
        "en": "",
        "ar": "اليهودية",
    },
    "sufi": {
        "male": "صوفي",
        "males": "صوفيون",
        "female": "صوفية",
        "females": "صوفيات",
        "the_male": "الصوفي",
        "the_female": "الصوفية",
        "en": "",
        "ar": "الصوفية",
    },
    "christian": {
        "male": "مسيحي",
        "males": "مسيحيون",
        "female": "مسيحية",
        "females": "مسيحيات",
        "the_male": "المسيحي",
        "the_female": "المسيحية",
        "en": "",
        "ar": "المسيحية",
    },
}


def load_sources(
    raw_nats_as_en_key: Dict[str, Dict[str, str]] | None = None,
) -> Dict[str, NationalityEntry]:
    """
    Load nationality JSON data and merge nationalities_data + sub_nats.
    Ensures all entries follow the NationalityEntry structure (string values only).

    """
    data_to_review = {
        "people from jerusalem": {
            "male": "مقدسي",
            "males": "مقدسيون",
            "female": "مقدسية",
            "females": "مقدسيات",
            "en": "jerusalem",
            "ar": "القدس",
            "the_female": "المقدسية",
            "the_male": "المقدسي",
        },
    }

    raw_all_nat_o: AllNatDict = open_json_file("nationalities/nationalities_data.json") or {}
    nationality_directions_mapping: AllNatDict = (
        open_json_file("nationalities/nationalities_data_with_directions.json") or {}
    )

    raw_uu_nats: AllNatDict = open_json_file("nationalities/sub_nats_with_ar_or_en.json") or {}
    raw_sub_nat: AllNatDict = open_json_file("nationalities/sub_nats.json") or {}
    continents: AllNatDict = open_json_file("nationalities/continents.json") or {}

    raw_all_nat_o.update(nationality_directions_mapping)

    if raw_nats_as_en_key:
        raw_all_nat_o.update(build_en_nat_entries(raw_nats_as_en_key))
        raw_all_nat_o.update(raw_nats_as_en_key)

    data = {}

    # Merge JSONs into nationalities_data
    data.update(raw_uu_nats)

    data.update(raw_sub_nat)
    data.update(continents)
    data.update(raw_sub_nat_additional)
    # for key, val in raw_sub_nat.items(): raw_all_nat_o[key] = val

    data.update(raw_all_nat_o)

    # Convert everything to NationalityEntry ensuring all fields exist
    normalized: Dict[str, NationalityEntry] = {}

    for key, val in data.items():
        # Build guaranteed structure
        val = val if isinstance(val, dict) else {}
        entry: NationalityEntry = build_nationality_structure(val)
        normalized[key] = entry

        # Special cases like "Category:Antigua and Barbuda writers" which use country names as nationalities
        en_key = entry["en"].lower()
        if en_key in countries_en_as_nationality_keys and en_key != key.lower():
            normalized[en_key] = entry

    return normalized


def build_en_nat_entries(raw_data: AllNatDict) -> AllNatDict:
    data: AllNatDict = {}
    if not raw_data:
        return {}
    for _, v in raw_data.items():
        if v.get("en_nat"):
            data[v["en_nat"]] = build_nationality_structure(v)

    return data


# =====================================================================
# Section 2: Normalize aliases
# =====================================================================


def normalize_aliases(all_nat_o: Dict[str, NationalityEntry], _print=False) -> Dict[str, NationalityEntry]:
    """
    Apply alias equivalence and one-off corrections.
    Ensures the resulting dictionary keys all map to NationalityEntry.
    """

    alias_map: Dict[str, str] = {
        "turkish cypriot": "northern cypriot",
        "luxembourg": "luxembourgish",
        "ancient romans": "ancient-romans",
        "ancient-roman": "ancient-romans",
        "arabian": "arab",
        "argentinean": "argentine",
        "argentinian": "argentine",
        "austro-hungarian": "austrianhungarian",
        "bangladesh": "bangladeshi",
        "barbadian_2": "barbadian",
        "belizian": "belizean",
        "bosnia and herzegovina": "bosnian",
        "burkinabé": "burkinabe",
        "burkinese": "burkinabe",
        "canadians": "canadian",
        "caribbean": "caribbeans",
        "comoran": "comorian",
        "democratic-republic-of-congo": "democratic republic of congo",
        "dominican republic": "dominican republic",
        "republic of congo": "republic of congo",
        "republic-of ireland": "irish",
        "republic-of-congo": "republic of congo",
        "emiri": "emirati",
        "emirian": "emirati",
        "equatorial guinean": "equatoguinean",
        "ivoirian": "ivorian",
        "kosovar": "kosovan",
        "lao": "laotian",
        "monacan": "monegasque",
        "monégasque": "monegasque",
        "mosotho": "lesotho",
        "nepali": "nepalese",
        "roman": "romanian",
        "russians": "russian",
        "salvadoran": "salvadorean",
        "saudi": "saudiarabian",
        "singapore": "singaporean",
        "slovakian": "slovak",
        "slovene": "slovenian",
        "somali": "somalian",
        "south ossetian": "ossetian",
        "trinidadian": "trinidad and tobago",
        "trinidadians": "trinidad and tobago",
        "vietnamesei": "vietnamese",
        "yemenite": "yemeni",
        "jewish": "jews",
        "native american": "native americans",
    }

    # Apply simple alias redirection
    for alias, target in alias_map.items():
        if alias == target:
            continue  # skip self-aliases

        if target in all_nat_o:
            all_nat_o[alias] = build_nationality_structure(all_nat_o[target])
        else:
            if _print:
                print(f"Alias({alias}) target ({target}) not found in nationality data")

    # NOTE: "papua new guinean" has same values as "guinean"
    all_nat_o["papua new guinean x"] = {
        "male": "غيني",
        "males": "غينيون",
        "female": "غينية",
        "females": "غينيات",
        "the_male": "الغيني",
        "the_female": "الغينية",
        "en": "papua new guinea",
        "ar": "بابوا غينيا الجديدة",
    }

    # Handle Georgia (country)
    if "georgian" in all_nat_o:
        all_nat_o["georgia (country)"] = build_nationality_structure(all_nat_o["georgian"])
        all_nat_o["georgia (country)"]["en"] = "georgia (country)"

    return all_nat_o


# =====================================================================
# Section 3: Build American forms
# =====================================================================


def build_american_forms(all_nat_o: Dict[str, NationalityEntry]) -> AllNatDict:
    """
    Build '-american' and 'x american' nationality forms.
    Returns: (updated_all_nat, count_of_added_items)
    """

    American_nat = {}

    for nat_key, entry in all_nat_o.items():
        male = entry["male"]
        males = entry["males"]
        female, females = entry["female"], entry["females"]

        if not any([male, males, female, females]):
            continue  # skip if no gender fields present

        the_female, the_male = entry.get("the_female", ""), entry.get("the_male", "")

        new_entry: NationalityEntry = {
            "male": f"أمريكي {male}" if male else "",
            "males": f"أمريكيون {males}" if males else "",
            "female": f"أمريكية {female}" if female else "",
            "females": f"أمريكيات {females}" if females else "",
            "en": "",
            "ar": "",
            "the_female": f"الأمريكية {the_female}" if the_female else "",
            "the_male": f"الأمريكي {the_male}" if the_male else "",
        }

        key_lower = nat_key.lower()
        American_nat[f"{key_lower}-american"] = new_entry

        # Special case
        if key_lower == "jewish":
            American_nat[f"{key_lower} american"] = new_entry

    return American_nat


# =====================================================================
# Section 4: Build lookup tables
# =====================================================================


def build_lookup_tables(all_nat: AllNatDict) -> Dict[str, Any]:
    """
    Build all nationality lookup tables used throughout the system.
    """

    Nat_men: LookupTable = {}
    Nat_mens: LookupTable = {}
    Nat_women: LookupTable = {}
    Nat_Womens: LookupTable = {}

    Nat_the_female: LookupTable = {}
    Nat_the_male: LookupTable = {}

    ar_Nat_men: LookupTable = {}
    countries_from_nat: LookupTable = {}

    all_country_ar: LookupTable = {}
    all_country_with_nat: AllNatDict = {}
    all_country_with_nat_ar: AllNatDict = {}
    countries_nat_en_key: Dict[str, NationalityEntry] = {}
    en_nats_to_ar_label: LookupTable = {}

    for nat_key, entry in all_nat.items():
        en: str = entry["en"].lower()
        ar: str = entry["ar"]
        en_norm: str = en[4:] if en.startswith("the ") else en

        if entry.get("male"):
            Nat_men[nat_key] = entry["male"]
            ar_Nat_men[entry["male"]] = nat_key

        if entry.get("males"):
            Nat_mens[nat_key] = entry["males"]

        if entry.get("female"):
            Nat_women[nat_key] = entry["female"]

        if entry.get("females"):
            Nat_Womens[nat_key] = entry["females"]

        if entry.get("the_female"):
            Nat_the_female[nat_key] = entry["the_female"]

        if entry.get("the_male"):
            Nat_the_male[nat_key] = entry["the_male"]

        # English → Arabic country mapping
        if en and ar:
            all_country_ar[en_norm] = ar
            countries_from_nat[en_norm] = ar

            if en_norm.startswith("the "):
                countries_from_nat[en_norm[4:]] = ar

        # Full nationality entry mapping
        if ar:
            all_country_with_nat_ar[nat_key] = entry
            en_nats_to_ar_label[nat_key] = ar

        if en:
            all_country_with_nat[nat_key] = entry
            countries_nat_en_key[en_norm] = entry

    # Special case: Iran
    if "iranian" in all_nat:
        countries_nat_en_key["islamic republic of iran"] = all_nat["iranian"]

    countries_from_nat.update(
        {
            "serbia and montenegro": "صربيا والجبل الأسود",
            "serbia-and-montenegro": "صربيا والجبل الأسود",
        }
    )

    return {
        "Nat_men": Nat_men,
        "Nat_mens": Nat_mens,
        "Nat_women": Nat_women,
        "Nat_Womens": Nat_Womens,
        "ar_Nat_men": ar_Nat_men,
        "countries_from_nat": countries_from_nat,
        "all_country_ar": all_country_ar,
        "all_country_with_nat": all_country_with_nat,
        "all_country_with_nat_ar": all_country_with_nat_ar,
        "countries_nat_en_key": countries_nat_en_key,
        "en_nats_to_ar_label": en_nats_to_ar_label,
        "Nat_the_male": Nat_the_male,
        "Nat_the_female": Nat_the_female,
    }


# =====================================================================
# Main Execution (same logic as before)
# =====================================================================

raw_nats_as_en_key: Dict[str, Any] = open_json_file("nationalities/all_nat_as_en.json") or {}

nationalities_data: Dict[str, NationalityEntry] = load_sources(raw_nats_as_en_key)

nationalities_data = normalize_aliases(nationalities_data, True)

All_Nat: AllNatDict = {k.lower(): v for k, v in nationalities_data.items()}

American_nat = build_american_forms(nationalities_data)
All_Nat.update(American_nat)
result_tables = build_lookup_tables(All_Nat)

Nat_men: LookupTable = result_tables["Nat_men"]
Nat_mens: LookupTable = result_tables["Nat_mens"]
Nat_women: LookupTable = result_tables["Nat_women"]
Nat_Womens: LookupTable = result_tables["Nat_Womens"]

Nat_the_male: LookupTable = result_tables["Nat_the_male"]
Nat_the_female: LookupTable = result_tables["Nat_the_female"]

ar_Nat_men: LookupTable = result_tables["ar_Nat_men"]
countries_from_nat: LookupTable = result_tables["countries_from_nat"]
all_country_ar: LookupTable = result_tables["all_country_ar"]

all_country_with_nat: AllNatDict = result_tables["all_country_with_nat"]
all_country_with_nat_ar: AllNatDict = result_tables["all_country_with_nat_ar"]
countries_nat_en_key: Dict[str, NationalityEntry] = result_tables["countries_nat_en_key"]

en_nats_to_ar_label: LookupTable = result_tables["en_nats_to_ar_label"]

nats_to_add = {}

all_nat_sorted = dict(
    sorted(
        All_Nat.items(),
        key=lambda k: (-k[0].count(" "), -len(k[0])),
    )
)

len_result = {
    "raw_nats_as_en_key": 17,
    "ar_Nat_men": 711,
    "Nat_men": 841,
    "Nat_mens": 843,
    "Nat_women": 843,
    "Nat_Womens": 843,
    "All_Nat": 843,
    "Nat_the_male": 843,
    "Nat_the_female": 843,
    "all_country_ar": 285,
    "countries_from_nat": 287,
    "all_country_with_nat_ar": 342,
    "en_nats_to_ar_label": 342,
    "all_country_with_nat": 336,
    "American_nat": 422,
    "countries_nat_en_key": 286,
}
len_print.data_len(
    "nationality.py",
    {
        "raw_nats_as_en_key": raw_nats_as_en_key,
        "ar_Nat_men": ar_Nat_men,
        "Nat_men": Nat_men,
        "Nat_mens": Nat_mens,
        "Nat_women": Nat_women,
        "Nat_Womens": Nat_Womens,
        "all_country_ar": all_country_ar,
        "countries_from_nat": countries_from_nat,
        "All_Nat": All_Nat,
        "all_country_with_nat_ar": all_country_with_nat_ar,
        "all_country_with_nat": all_country_with_nat,
        "American_nat": American_nat,
        "countries_nat_en_key": countries_nat_en_key,
        "en_nats_to_ar_label": en_nats_to_ar_label,
        "Nat_the_male": Nat_the_male,
        "Nat_the_female": Nat_the_female,
    },
)



"""Shared utilities for the :mod:`make_bots.o_bots` package."""

from __future__ import annotations

import re
from typing import Callable, Mapping, Optional, Sequence, Tuple

from ...helps import logger

ValueLookup = Callable[[str], str]


def match_suffix_template(name: str, suffixes: Mapping[str, str]) -> Optional[Tuple[str, str]]:
    """
    Find the first suffix template that matches ``name``.

    input: 'football governing bodies'
    output: prefix='football governing' -> template='هيئات {}'
    """

    stripped = name.strip()
    # sorted by len of " " in key
    sorted_suffixes = dict(
        sorted(
            suffixes.items(),
            key=lambda k: (-k[0].count(" "), -len(k[0])),
        )
    )

    for suffix, template in sorted_suffixes.items():
        candidates = [suffix]
        if not suffix.startswith(" "):
            candidates.append(f" {suffix}")

        for candidate in candidates:
            if stripped.endswith(candidate):
                prefix = stripped[: -len(candidate)].strip()
                logger.debug(f"match_suffix_template: {name=} -> {candidate=} -> {prefix=}")
                return prefix, template
    return None


def resolve_suffix_template(name: str, suffix_templates: Mapping[str, str], lookup: ValueLookup) -> str:
    """Resolve ``name`` using ``suffix_templates`` and ``lookup``."""

    match = match_suffix_template(name, suffix_templates)
    if not match:
        return ""

    prefix, template = match

    lookup_value = lookup(prefix)
    logger.debug(f"resolve_suffix_template: {prefix=} -> {lookup_value=}")

    if not lookup_value:
        return ""

    result = template % lookup_value if "%s" in template else template.format(lookup_value)
    logger.debug(f"resolve_suffix_template: {result=}")

    return result


def first_non_empty(key: str, tables: Sequence[Mapping[str, str]]) -> str:
    """Return the first non-empty label for ``key`` from ``tables``."""

    for table in tables:
        value = table.get(key, "")
        if value:
            return value
    return ""


def apply_arabic_article(label: str) -> str:
    """Prefix ``label`` with the Arabic definite article for each word."""

    if not label:
        return ""

    added = "ال"
    article_applied = re.sub(r" ", f" {added}", label)

    result = f"{added}{article_applied}".strip()
    result = result.replace(f" {added}{added}", f" {added}")

    return result



#
import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_arabic_category_label

examples = {
    "Category:18th-century Dutch explorers": "تصنيف:مستكشفون هولنديون في القرن 18",
    "Category:20th-century Albanian sports coaches": "تصنيف:مدربو رياضة ألبان في القرن 20",
    "Category:19th-century actors": "تصنيف:ممثلون في القرن 19",
    "Category:2000s American films": "تصنيف:أفلام أمريكية في عقد 2000",
    "Category:2017 American television series debuts": "تصنيف:مسلسلات تلفزيونية أمريكية بدأ عرضها في 2017",
    "Category:2017 American television series endings": "تصنيف:مسلسلات تلفزيونية أمريكية انتهت في 2017",
    "Category:19th-century actors by religion": "تصنيف:ممثلون في القرن 19 حسب الدين",
    "Category:19th-century people by religion": "تصنيف:أشخاص في القرن 19 حسب الدين",
}

TEMPORAL_CASES = [
    ("temporal_1", examples),
]


@pytest.mark.parametrize("name,data", TEMPORAL_CASES)
@pytest.mark.skip2
def test_temporal_add_in(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"


@pytest.mark.parametrize(
    "category, expected",
    examples.items(),
    ids=[k for k in examples],
)
def test_add_in(category: str, expected: str) -> None:
    assert resolve_arabic_category_label(category) == expected
